@@ -2,8 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2021 The Fluent Bit Authors
- *  Copyright (C) 2015-2018 Treasure Data Inc.
+ *  Copyright (C) 2015-2024 The Fluent Bit Authors
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -40,6 +39,7 @@ struct flb_ml_parser *flb_ml_parser_create(struct flb_config *ctx,
 int flb_ml_parser_destroy(struct flb_ml_parser *ml_parser);
 void flb_ml_parser_destroy_all(struct mk_list *list);
 
+struct flb_ml_parser *flb_ml_parser_get(struct flb_config *ctx, char *name);
 
 struct flb_ml_parser_ins *flb_ml_parser_instance_create(struct flb_ml *ml,
                                                         char *name);
@@ -53,6 +53,7 @@ struct flb_ml_parser *flb_ml_parser_docker(struct flb_config *config);
 struct flb_ml_parser *flb_ml_parser_cri(struct flb_config *config);
 struct flb_ml_parser *flb_ml_parser_java(struct flb_config *config, char *key);
 struct flb_ml_parser *flb_ml_parser_go(struct flb_config *config, char *key);
+struct flb_ml_parser *flb_ml_parser_ruby(struct flb_config *config, char *key);
 struct flb_ml_parser *flb_ml_parser_python(struct flb_config *config, char *key);
 
 #endif
