@@ -1,17 +1,25 @@
-/*
- * Copyright (C) The c-ares project
+/* MIT License
  *
- * Permission to use, copy, modify, and distribute this
- * software and its documentation for any purpose and without
- * fee is hereby granted, provided that the above copyright
- * notice appear in all copies and that both that copyright
- * notice and this permission notice appear in supporting
- * documentation, and that the name of M.I.T. not be used in
- * advertising or publicity pertaining to distribution of the
- * software without specific, written prior permission.
- * M.I.T. makes no representations about the suitability of
- * this software for any purpose.  It is provided "as is"
- * without express or implied warranty.
+ * Copyright (c) The c-ares project and its contributors
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to deal
+ * in the Software without restriction, including without limitation the rights
+ * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice (including the next
+ * paragraph) shall be included in all copies or substantial portions of the
+ * Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  *
  * SPDX-License-Identifier: MIT
  */
@@ -56,7 +64,7 @@ TEST_F(DefaultChannelTest, SetServers) {
   ares_gethostbyname(channel_, "www.google.com.", AF_INET, HostCallback, &result);
   Process();
   EXPECT_TRUE(result.done_);
-  EXPECT_EQ(ARES_ESERVFAIL, result.status_);
+  EXPECT_EQ(ARES_ENOSERVER, result.status_);
 
 
   struct ares_addr_node server1;
@@ -96,7 +104,7 @@ TEST_F(DefaultChannelTest, SetServersPorts) {
   server2.family = AF_INET;
   server2.addr.addr4.s_addr = htonl(0x02030405);
   server2.udp_port = 0;
-  server2.tcp_port = 0;;
+  server2.tcp_port = 0;
   EXPECT_EQ(ARES_ENODATA, ares_set_servers_ports(nullptr, &server1));
 
   EXPECT_EQ(ARES_SUCCESS, ares_set_servers_ports(channel_, &server1));
@@ -300,6 +308,9 @@ TEST_F(LibraryTest, CreateQueryFailures) {
             ares_create_query("example..com", C_IN, T_A, 0x1234, 0,
                     &p, &len, 0));
   if (p) ares_free_string(p);
+
+  EXPECT_EQ(ARES_EFORMERR,
+            ares_create_query(NULL, C_IN, T_A, 0x1234, 0, NULL, NULL, 0));
 }
 
 TEST_F(LibraryTest, CreateQueryOnionDomain) {
@@ -351,8 +362,8 @@ TEST_F(DefaultChannelTest, SendFailure) {
   EXPECT_EQ(ARES_EBADQUERY, result.status_);
 }
 
-std::string ExpandName(const std::vector<byte>& data, int offset,
-                       long *enclen) {
+static std::string ExpandName(const std::vector<byte>& data, int offset,
+                              long *enclen) {
   char *name = nullptr;
   int rc = ares_expand_name(data.data() + offset, data.data(), (int)data.size(),
                             &name, enclen);
@@ -536,15 +547,6 @@ TEST_F(LibraryTest, Version) {
   EXPECT_EQ(ARES_VERSION, version);
 }
 
-TEST_F(LibraryTest, Strerror) {
-  EXPECT_EQ("Successful completion",
-            std::string(ares_strerror(ARES_SUCCESS)));
-  EXPECT_EQ("DNS query cancelled",
-            std::string(ares_strerror(ARES_ECANCELLED)));
-  EXPECT_EQ("unknown",
-            std::string(ares_strerror(99)));
-}
-
 TEST_F(LibraryTest, ExpandString) {
   std::vector<byte> s1 = { 3, 'a', 'b', 'c'};
   char* result = nullptr;
@@ -568,5 +570,90 @@ TEST_F(LibraryTest, ExpandString) {
                                (unsigned char**)&result, &len));
 }
 
+TEST_F(LibraryTest, DNSMapping) {
+  ares_dns_rec_type_t types[] = {
+    ARES_REC_TYPE_A,
+    ARES_REC_TYPE_NS,
+    ARES_REC_TYPE_CNAME,
+    ARES_REC_TYPE_SOA,
+    ARES_REC_TYPE_PTR,
+    ARES_REC_TYPE_HINFO,
+    ARES_REC_TYPE_MX,
+    ARES_REC_TYPE_TXT,
+    ARES_REC_TYPE_SIG,
+    ARES_REC_TYPE_AAAA,
+    ARES_REC_TYPE_SRV,
+    ARES_REC_TYPE_NAPTR,
+    ARES_REC_TYPE_OPT,
+    ARES_REC_TYPE_TLSA,
+    ARES_REC_TYPE_SVCB,
+    ARES_REC_TYPE_HTTPS,
+    ARES_REC_TYPE_ANY,
+    ARES_REC_TYPE_URI,
+    ARES_REC_TYPE_CAA
+  };
+
+  for (size_t i=0; i<sizeof(types) / sizeof(*types); i++) {
+    ares_dns_rec_type_t type;
+    EXPECT_TRUE(ares_dns_rec_type_fromstr(&type, ares_dns_rec_type_tostr(types[i])));
+    EXPECT_EQ(types[i], type);
+    size_t cnt;
+    const ares_dns_rr_key_t *keys = ares_dns_rr_get_keys(type, &cnt);
+    for (size_t j=0; j<cnt; j++) {
+      const char *name = ares_dns_rr_key_tostr(keys[j]);
+      EXPECT_NE(nullptr, name);
+      EXPECT_NE("UNKNOWN", std::string(name));
+      EXPECT_EQ(type, ares_dns_rr_key_to_rec_type(keys[j]));
+      EXPECT_NE(0, (int)ares_dns_rr_key_datatype(keys[j]));
+    }
+  }
+}
+
+TEST_F(LibraryTest, StrError) {
+  ares_status_t status[] = {
+    ARES_SUCCESS, ARES_ENODATA, ARES_EFORMERR, ARES_ESERVFAIL, ARES_ENOTFOUND,
+    ARES_ENOTIMP, ARES_EREFUSED, ARES_EBADQUERY, ARES_EBADNAME, ARES_EBADFAMILY,
+    ARES_EBADRESP, ARES_ECONNREFUSED, ARES_ETIMEOUT, ARES_EOF, ARES_EFILE,
+    ARES_ENOMEM, ARES_EDESTRUCTION, ARES_EBADSTR, ARES_EBADFLAGS, ARES_ENONAME,
+    ARES_EBADHINTS, ARES_ENOTINITIALIZED, ARES_ELOADIPHLPAPI,
+    ARES_EADDRGETNETWORKPARAMS, ARES_ECANCELLED, ARES_ESERVICE, ARES_ENOSERVER
+  };
+  size_t i;
+  const char *str = nullptr;
+
+  for (i=0; i < sizeof(status) / sizeof(*status); i++) {
+    str = ares_strerror((int)status[i]);
+    EXPECT_NE(nullptr, str);
+    EXPECT_NE("unknown", std::string(str));
+  }
+
+  /* unknown value */
+  str = ares_strerror(0x12345678);
+  EXPECT_NE(nullptr, str);
+  EXPECT_EQ("unknown", std::string(str));
+}
+
+TEST_F(LibraryTest, UsageErrors) {
+  ares_cancel(NULL);
+  ares_set_socket_callback(NULL, NULL, NULL);
+  ares_set_socket_configure_callback(NULL, NULL, NULL);
+  ares_set_socket_functions(NULL, NULL, NULL);
+  ares_destroy(NULL);
+  ares_expand_name(NULL, NULL, 0, NULL, NULL);
+  ares_expand_string(NULL, NULL, 0, NULL, NULL);
+  ares_fds(NULL, NULL, NULL);
+  ares_getaddrinfo(NULL, NULL, NULL, NULL, NULL, NULL);
+  ares_gethostbyaddr(NULL, NULL, 0, 0, NULL, NULL);
+  ares_getnameinfo(NULL, NULL, 0, 0, NULL, NULL);
+  ares_reinit(NULL);
+  ares_dup(NULL, NULL);
+  ares_set_local_ip4(NULL, 0);
+  ares_set_local_ip6(NULL, NULL);
+  ares_set_local_dev(NULL, NULL);
+  ares_query_dnsrec(NULL, NULL, ARES_CLASS_IN, ARES_REC_TYPE_A, NULL, NULL, NULL);
+  ares_query(NULL, NULL, ARES_CLASS_IN, ARES_REC_TYPE_A, NULL, NULL);
+}
+
+
 }  // namespace test
 }  // namespace ares
