@@ -1,17 +1,25 @@
-/*
- * Copyright (C) The c-ares project
+/* MIT License
  *
- * Permission to use, copy, modify, and distribute this
- * software and its documentation for any purpose and without
- * fee is hereby granted, provided that the above copyright
- * notice appear in all copies and that both that copyright
- * notice and this permission notice appear in supporting
- * documentation, and that the name of M.I.T. not be used in
- * advertising or publicity pertaining to distribution of the
- * software without specific, written prior permission.
- * M.I.T. makes no representations about the suitability of
- * this software for any purpose.  It is provided "as is"
- * without express or implied warranty.
+ * Copyright (c) The c-ares project and its contributors
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to deal
+ * in the Software without restriction, including without limitation the rights
+ * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice (including the next
+ * paragraph) shall be included in all copies or substantial portions of the
+ * Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  *
  * SPDX-License-Identifier: MIT
  */
@@ -32,6 +40,7 @@ namespace ares {
 namespace test {
 
 MATCHER_P(IncludesNumAddresses, n, "") {
+  (void)result_listener;
   if(!arg)
     return false;
   int cnt = 0;
@@ -41,6 +50,7 @@ MATCHER_P(IncludesNumAddresses, n, "") {
 }
 
 MATCHER_P(IncludesV4Address, address, "") {
+  (void)result_listener;
   if(!arg)
     return false;
   in_addr addressnum = {};
@@ -59,6 +69,7 @@ MATCHER_P(IncludesV4Address, address, "") {
 }
 
 MATCHER_P(IncludesV6Address, address, "") {
+  (void)result_listener;
   if(!arg)
     return false;
   in6_addr addressnum = {};
@@ -346,6 +357,53 @@ TEST_P(MockExtraOptsNDots5TestAI, SimpleQuery) {
   EXPECT_THAT(result.ai_, IncludesV4Address("123.45.67.8"));
 }
 
+class MockExtraOptsNDots0TestAI : public MockExtraOptsNDotsTestAI {
+ public:
+  MockExtraOptsNDots0TestAI() : MockExtraOptsNDotsTestAI(0) {}
+};
+
+TEST_P(MockExtraOptsNDots0TestAI, SimpleQuery) {
+  DNSPacket rsp_ndots0;
+  rsp_ndots0.set_response().set_aa()
+    .add_question(new DNSQuestion("ndots0", T_A))
+    .add_answer(new DNSARR("ndots0", 100, {1, 2, 3, 4}));
+  ON_CALL(server_, OnRequest("ndots0", T_A))
+    .WillByDefault(SetReply(&server_, &rsp_ndots0));
+
+  DNSPacket rsp_ndots0_first;
+  rsp_ndots0_first.set_response().set_aa()
+    .add_question(new DNSQuestion("ndots0.first.com", T_A))
+    .add_answer(new DNSARR("ndots0.first.com", 100, {99, 99, 99, 99}));
+  ON_CALL(server_, OnRequest("ndots0.first.com", T_A))
+    .WillByDefault(SetReply(&server_, &rsp_ndots0_first));
+
+  DNSPacket rsp_ndots0_second;
+  rsp_ndots0_second.set_response().set_aa()
+    .add_question(new DNSQuestion("ndots0.second.org", T_A))
+    .add_answer(new DNSARR("ndots0.second.org", 100, {98, 98, 98, 98}));
+  ON_CALL(server_, OnRequest("ndots0.second.org", T_A))
+    .WillByDefault(SetReply(&server_, &rsp_ndots0_second));
+
+  DNSPacket rsp_ndots0_third;
+  rsp_ndots0_third.set_response().set_aa()
+    .add_question(new DNSQuestion("ndots0.third.gov", T_A))
+    .add_answer(new DNSARR("ndots0.third.gov", 100, {97, 97, 97, 97}));
+  ON_CALL(server_, OnRequest("ndots0.third.gov", T_A))
+    .WillByDefault(SetReply(&server_, &rsp_ndots0_third));
+
+  AddrInfoResult result;
+  struct ares_addrinfo_hints hints = {};
+  hints.ai_family = AF_INET;
+  hints.ai_flags = ARES_AI_NOSORT;
+  ares_getaddrinfo(channel_, "ndots0", NULL, &hints, AddrInfoCallback, &result);
+  Process();
+  EXPECT_TRUE(result.done_);
+  EXPECT_EQ(ARES_SUCCESS, result.status_);
+  std::stringstream ss;
+  ss << result.ai_;
+  EXPECT_EQ("{addr=[1.2.3.4]}", ss.str());
+}
+
 class MockFlagsChannelOptsTestAI
     : public MockChannelOptsTest,
       public ::testing::WithParamInterface< std::pair<int, bool> > {
@@ -442,6 +500,7 @@ TEST_P(MockChannelTestAI, FamilyV6) {
   EXPECT_THAT(result.ai_, IncludesV6Address("2121:0000:0000:0000:0000:0000:0000:0303"));
 }
 
+#ifndef CARES_SYMBOL_HIDING
 // Test case for Issue #662
 TEST_P(MockChannelTestAI, PartialQueryCancel) {
   std::vector<byte> nothing;
@@ -469,6 +528,7 @@ TEST_P(MockChannelTestAI, PartialQueryCancel) {
   EXPECT_TRUE(result.done_);
   EXPECT_EQ(ARES_ECANCELLED, result.status_);
 }
+#endif
 
 TEST_P(MockChannelTestAI, FamilyV4) {
   DNSPacket rsp4;
@@ -645,8 +705,8 @@ class MockMultiServerChannelTestAI
   : public MockChannelOptsTest,
     public ::testing::WithParamInterface< std::pair<int, bool> > {
  public:
-  MockMultiServerChannelTestAI(bool rotate)
-    : MockChannelOptsTest(3, GetParam().first, GetParam().second, nullptr, rotate ? ARES_OPT_ROTATE : ARES_OPT_NOROTATE) {}
+  MockMultiServerChannelTestAI(ares_options *opts, int optmask)
+    : MockChannelOptsTest(3, GetParam().first, GetParam().second, opts, optmask) {}
   void CheckExample() {
     AddrInfoResult result;
     struct ares_addrinfo_hints hints = {};
@@ -663,7 +723,22 @@ class MockMultiServerChannelTestAI
 
 class NoRotateMultiMockTestAI : public MockMultiServerChannelTestAI {
  public:
-  NoRotateMultiMockTestAI() : MockMultiServerChannelTestAI(false) {}
+  NoRotateMultiMockTestAI() : MockMultiServerChannelTestAI(nullptr, ARES_OPT_NOROTATE) {}
+};
+
+class ServerFailoverOptsMockTestAI : public MockMultiServerChannelTestAI {
+ public:
+  ServerFailoverOptsMockTestAI()
+    : MockMultiServerChannelTestAI(FillOptions(&opts_),
+                                   ARES_OPT_SERVER_FAILOVER | ARES_OPT_NOROTATE) {}
+  static struct ares_options* FillOptions(struct ares_options *opts) {
+    memset(opts, 0, sizeof(struct ares_options));
+    opts->server_failover_opts.retry_chance = 1;
+    opts->server_failover_opts.retry_delay = 250;
+    return opts;
+  }
+ private:
+  struct ares_options opts_;
 };
 
 TEST_P(NoRotateMultiMockTestAI, ThirdServer) {
@@ -717,6 +792,89 @@ TEST_P(NoRotateMultiMockTestAI, ThirdServer) {
   CheckExample();
 }
 
+// Test case to trigger server failover behavior. We use a retry chance of
+// 100% and a retry delay of 250ms so that we can test behavior reliably.
+TEST_P(ServerFailoverOptsMockTestAI, ServerFailoverOpts) {
+  DNSPacket servfailrsp;
+  servfailrsp.set_response().set_aa().set_rcode(SERVFAIL)
+    .add_question(new DNSQuestion("www.example.com", T_A));
+  DNSPacket okrsp;
+  okrsp.set_response().set_aa()
+    .add_question(new DNSQuestion("www.example.com", T_A))
+    .add_answer(new DNSARR("www.example.com", 100, {2,3,4,5}));
+
+  // 1. If all servers are healthy, then the first server should be selected.
+  EXPECT_CALL(*servers_[0], OnRequest("www.example.com", T_A))
+    .WillOnce(SetReply(servers_[0].get(), &okrsp));
+  CheckExample();
+
+  // 2. Failed servers should be retried after the retry delay.
+  //
+  // Fail server #0 but leave server #1 as healthy.
+  EXPECT_CALL(*servers_[0], OnRequest("www.example.com", T_A))
+    .WillOnce(SetReply(servers_[0].get(), &servfailrsp));
+  EXPECT_CALL(*servers_[1], OnRequest("www.example.com", T_A))
+    .WillOnce(SetReply(servers_[1].get(), &okrsp));
+  CheckExample();
+
+  // Sleep for the retry delay (actually a little more than 250ms to account
+  // for unreliable timing, e.g. NTP slew) and send in another query. Server #0
+  // should be retried.
+  std::this_thread::sleep_for(std::chrono::milliseconds(260));
+  EXPECT_CALL(*servers_[0], OnRequest("www.example.com", T_A))
+    .WillOnce(SetReply(servers_[0].get(), &okrsp));
+  CheckExample();
+
+  // 3. If there are multiple failed servers, then the servers should be
+  //    retried in sorted order.
+  //
+  // Fail all servers for the first round of tries. On the second round server
+  // #1 responds successfully.
+  EXPECT_CALL(*servers_[0], OnRequest("www.example.com", T_A))
+    .WillOnce(SetReply(servers_[0].get(), &servfailrsp))
+    .WillOnce(SetReply(servers_[0].get(), &servfailrsp));
+  EXPECT_CALL(*servers_[1], OnRequest("www.example.com", T_A))
+    .WillOnce(SetReply(servers_[1].get(), &servfailrsp))
+    .WillOnce(SetReply(servers_[1].get(), &okrsp));
+  EXPECT_CALL(*servers_[2], OnRequest("www.example.com", T_A))
+    .WillOnce(SetReply(servers_[2].get(), &servfailrsp));
+  CheckExample();
+
+  // At this point the sorted servers look like [1] (f0) [2] (f1) [0] (f2).
+  // Sleep for the retry delay and send in another query. Server #2 should be
+  // retried first, and then server #0.
+  std::this_thread::sleep_for(std::chrono::milliseconds(260));
+  EXPECT_CALL(*servers_[2], OnRequest("www.example.com", T_A))
+    .WillOnce(SetReply(servers_[2].get(), &servfailrsp));
+  EXPECT_CALL(*servers_[0], OnRequest("www.example.com", T_A))
+    .WillOnce(SetReply(servers_[0].get(), &okrsp));
+  CheckExample();
+
+  // 4. If there are multiple failed servers, then servers which have not yet
+  //    met the retry delay should be skipped.
+  //
+  // The sorted servers currently look like [0] (f0) [1] (f0) [2] (f2) and
+  // server #2 has just been retried.
+  // Sleep for half the retry delay and trigger a failure on server #0.
+  std::this_thread::sleep_for(std::chrono::milliseconds(130));
+  EXPECT_CALL(*servers_[0], OnRequest("www.example.com", T_A))
+    .WillOnce(SetReply(servers_[0].get(), &servfailrsp));
+  EXPECT_CALL(*servers_[1], OnRequest("www.example.com", T_A))
+    .WillOnce(SetReply(servers_[1].get(), &okrsp));
+  CheckExample();
+
+  // The sorted servers now look like [1] (f0) [0] (f1) [2] (f2). Server #0
+  // has just failed whilst server #2 is halfway through the retry delay.
+  // Sleep for another half the retry delay and check that server #2 is retried
+  // whilst server #0 is not.
+  std::this_thread::sleep_for(std::chrono::milliseconds(130));
+  EXPECT_CALL(*servers_[2], OnRequest("www.example.com", T_A))
+    .WillOnce(SetReply(servers_[2].get(), &servfailrsp));
+  EXPECT_CALL(*servers_[1], OnRequest("www.example.com", T_A))
+    .WillOnce(SetReply(servers_[1].get(), &okrsp));
+  CheckExample();
+}
+
 TEST_P(MockChannelTestAI, FamilyV4ServiceName) {
   DNSPacket rsp4;
   rsp4.set_response().set_aa()
@@ -738,29 +896,34 @@ TEST_P(MockChannelTestAI, FamilyV4ServiceName) {
 }
 
 INSTANTIATE_TEST_SUITE_P(AddressFamiliesAI, MockChannelTestAI,
-                       ::testing::ValuesIn(ares::test::families_modes));
+                       ::testing::ValuesIn(ares::test::families_modes), PrintFamilyMode);
 
 INSTANTIATE_TEST_SUITE_P(AddressFamiliesAI, MockUDPChannelTestAI,
-                        ::testing::ValuesIn(ares::test::families));
+                        ::testing::ValuesIn(ares::test::families), PrintFamily);
 
 INSTANTIATE_TEST_SUITE_P(AddressFamiliesAI, MockTCPChannelTestAI,
-                        ::testing::ValuesIn(ares::test::families));
+                        ::testing::ValuesIn(ares::test::families), PrintFamily);
 
 INSTANTIATE_TEST_SUITE_P(AddressFamiliesAI, MockExtraOptsTestAI,
-			::testing::ValuesIn(ares::test::families_modes));
+			::testing::ValuesIn(ares::test::families_modes), PrintFamilyMode);
 
 INSTANTIATE_TEST_SUITE_P(AddressFamiliesAI, MockExtraOptsNDots5TestAI,
-      ::testing::ValuesIn(ares::test::families_modes));
+      ::testing::ValuesIn(ares::test::families_modes), PrintFamilyMode);
+
+INSTANTIATE_TEST_SUITE_P(AddressFamiliesAI, MockExtraOptsNDots0TestAI,
+      ::testing::ValuesIn(ares::test::families_modes), PrintFamilyMode);
 
 INSTANTIATE_TEST_SUITE_P(AddressFamiliesAI, MockNoCheckRespChannelTestAI,
-			::testing::ValuesIn(ares::test::families_modes));
+			::testing::ValuesIn(ares::test::families_modes), PrintFamilyMode);
 
 INSTANTIATE_TEST_SUITE_P(AddressFamiliesAI, MockEDNSChannelTestAI,
-			::testing::ValuesIn(ares::test::families_modes));
+			::testing::ValuesIn(ares::test::families_modes), PrintFamilyMode);
 
 INSTANTIATE_TEST_SUITE_P(TransportModesAI, NoRotateMultiMockTestAI,
-			::testing::ValuesIn(ares::test::families_modes));
+			::testing::ValuesIn(ares::test::families_modes), PrintFamilyMode);
 
+INSTANTIATE_TEST_SUITE_P(TransportModesAI, ServerFailoverOptsMockTestAI,
+			::testing::ValuesIn(ares::test::families_modes), PrintFamilyMode);
 
 }  // namespace test
 }  // namespace ares
