@@ -54,9 +54,11 @@ extern "C" {
 #endif
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 
 #include <functional>
 #include <sstream>
+#include <algorithm>
 
 #ifdef WIN32
 #define BYTE_CAST (char *)
@@ -92,12 +94,161 @@ const std::vector<std::pair<int, bool>> ipv6_family_both_modes = {
   std::make_pair<int, bool>(AF_INET6, true)
 };
 
+
+const std::vector<std::tuple<ares_evsys_t, int, bool>> all_evsys_ipv4_family_both_modes = {
+#ifdef _WIN32
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_WIN32, AF_INET, false),
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_WIN32, AF_INET, true),
+#endif
+#ifdef HAVE_KQUEUE
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_KQUEUE, AF_INET, false),
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_KQUEUE, AF_INET, true),
+#endif
+#ifdef HAVE_EPOLL
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_EPOLL, AF_INET, false),
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_EPOLL, AF_INET, true),
+#endif
+#ifdef HAVE_POLL
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_POLL, AF_INET, false),
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_POLL, AF_INET, true),
+#endif
+#ifdef HAVE_PIPE
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_SELECT, AF_INET, false),
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_SELECT, AF_INET, true),
+#endif
+};
+
+const std::vector<std::tuple<ares_evsys_t, int, bool>> all_evsys_ipv6_family_both_modes = {
+#ifdef _WIN32
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_WIN32, AF_INET6, false),
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_WIN32, AF_INET6, true),
+#endif
+#ifdef HAVE_KQUEUE
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_KQUEUE, AF_INET6, false),
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_KQUEUE, AF_INET6, true),
+#endif
+#ifdef HAVE_EPOLL
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_EPOLL, AF_INET6, false),
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_EPOLL, AF_INET6, true),
+#endif
+#ifdef HAVE_POLL
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_POLL, AF_INET6, false),
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_POLL, AF_INET6, true),
+#endif
+#ifdef HAVE_PIPE
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_SELECT, AF_INET6, false),
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_SELECT, AF_INET6, true),
+#endif
+};
+
+const std::vector<std::tuple<ares_evsys_t, int, bool>> all_evsys_both_families_both_modes = {
+#ifdef _WIN32
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_WIN32, AF_INET, false),
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_WIN32, AF_INET, true),
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_WIN32, AF_INET6, false),
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_WIN32, AF_INET6, true),
+#endif
+#ifdef HAVE_KQUEUE
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_KQUEUE, AF_INET, false),
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_KQUEUE, AF_INET, true),
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_KQUEUE, AF_INET6, false),
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_KQUEUE, AF_INET6, true),
+#endif
+#ifdef HAVE_EPOLL
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_EPOLL, AF_INET, false),
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_EPOLL, AF_INET, true),
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_EPOLL, AF_INET6, false),
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_EPOLL, AF_INET6, true),
+#endif
+#ifdef HAVE_POLL
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_POLL, AF_INET, false),
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_POLL, AF_INET, true),
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_POLL, AF_INET6, false),
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_POLL, AF_INET6, true),
+#endif
+#ifdef HAVE_PIPE
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_SELECT, AF_INET, false),
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_SELECT, AF_INET, true),
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_SELECT, AF_INET6, false),
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_SELECT, AF_INET6, true),
+#endif
+};
+
+
+std::vector<std::tuple<ares_evsys_t, int, bool>> evsys_families_modes = all_evsys_both_families_both_modes;
+
+
+const std::vector<std::tuple<ares_evsys_t, int>> all_evsys_ipv4_family = {
+#ifdef _WIN32
+  std::make_tuple<ares_evsys_t, int>(ARES_EVSYS_WIN32, AF_INET),
+#endif
+#ifdef HAVE_KQUEUE
+  std::make_tuple<ares_evsys_t, int>(ARES_EVSYS_KQUEUE, AF_INET),
+#endif
+#ifdef HAVE_EPOLL
+  std::make_tuple<ares_evsys_t, int>(ARES_EVSYS_EPOLL, AF_INET),
+#endif
+#ifdef HAVE_POLL
+  std::make_tuple<ares_evsys_t, int>(ARES_EVSYS_POLL, AF_INET),
+#endif
+#ifdef HAVE_PIPE
+  std::make_tuple<ares_evsys_t, int>(ARES_EVSYS_SELECT, AF_INET),
+#endif
+};
+
+const std::vector<std::tuple<ares_evsys_t, int>> all_evsys_ipv6_family = {
+#ifdef _WIN32
+  std::make_tuple<ares_evsys_t, int>(ARES_EVSYS_WIN32, AF_INET6),
+#endif
+#ifdef HAVE_KQUEUE
+  std::make_tuple<ares_evsys_t, int>(ARES_EVSYS_KQUEUE, AF_INET6),
+#endif
+#ifdef HAVE_EPOLL
+  std::make_tuple<ares_evsys_t, int>(ARES_EVSYS_EPOLL, AF_INET6),
+#endif
+#ifdef HAVE_POLL
+  std::make_tuple<ares_evsys_t, int>(ARES_EVSYS_POLL, AF_INET6),
+#endif
+#ifdef HAVE_PIPE
+  std::make_tuple<ares_evsys_t, int>(ARES_EVSYS_SELECT, AF_INET6),
+#endif
+};
+
+const std::vector<std::tuple<ares_evsys_t, int>> all_evsys_both_families = {
+#ifdef _WIN32
+  std::make_tuple<ares_evsys_t, int>(ARES_EVSYS_WIN32, AF_INET),
+  std::make_tuple<ares_evsys_t, int>(ARES_EVSYS_WIN32, AF_INET6),
+#endif
+#ifdef HAVE_KQUEUE
+  std::make_tuple<ares_evsys_t, int>(ARES_EVSYS_KQUEUE, AF_INET),
+  std::make_tuple<ares_evsys_t, int>(ARES_EVSYS_KQUEUE, AF_INET6),
+#endif
+#ifdef HAVE_EPOLL
+  std::make_tuple<ares_evsys_t, int>(ARES_EVSYS_EPOLL, AF_INET),
+  std::make_tuple<ares_evsys_t, int>(ARES_EVSYS_EPOLL, AF_INET6),
+#endif
+#ifdef HAVE_POLL
+  std::make_tuple<ares_evsys_t, int>(ARES_EVSYS_POLL, AF_INET),
+  std::make_tuple<ares_evsys_t, int>(ARES_EVSYS_POLL, AF_INET6),
+#endif
+#ifdef HAVE_PIPE
+  std::make_tuple<ares_evsys_t, int>(ARES_EVSYS_SELECT, AF_INET),
+  std::make_tuple<ares_evsys_t, int>(ARES_EVSYS_SELECT, AF_INET6),
+#endif
+};
+
+
+
+std::vector<std::tuple<ares_evsys_t, int>> evsys_families = all_evsys_both_families;
+
+
 // Which parameters to use in tests
 std::vector<int> families = both_families;
 std::vector<std::pair<int, bool>> families_modes = both_families_both_modes;
 
 unsigned long long LibraryTest::fails_ = 0;
 std::map<size_t, int> LibraryTest::size_fails_;
+std::mutex            LibraryTest::lock_;
 
 void ProcessWork(ares_channel_t *channel,
                  std::function<std::set<ares_socket_t>()> get_extrafds,
@@ -105,18 +256,33 @@ void ProcessWork(ares_channel_t *channel,
                  unsigned int cancel_ms) {
   int nfds, count;
   fd_set readers, writers;
-  struct timeval tv_begin  = ares__tvnow();
-  struct timeval tv_cancel = tv_begin;
+
+#ifndef CARES_SYMBOL_HIDING
+  ares_timeval_t tv_begin;
+  ares_timeval_t tv_cancel;
+
+  ares__tvnow(&tv_begin);
+  memcpy(&tv_cancel, &tv_begin, sizeof(tv_cancel));
 
   if (cancel_ms) {
     if (verbose) std::cerr << "ares_cancel will be called after " << cancel_ms << "ms" << std::endl;
-    tv_cancel.tv_sec  += (cancel_ms / 1000);
-    tv_cancel.tv_usec += ((cancel_ms % 1000) * 1000);
+    tv_cancel.sec  += (cancel_ms / 1000);
+    tv_cancel.usec += ((cancel_ms % 1000) * 1000);
   }
+#else
+  if (cancel_ms) {
+    std::cerr << "library built with symbol hiding, can't test with cancel support" << std::endl;
+    return;
+  }
+#endif
 
   while (true) {
-    struct timeval  tv_now = ares__tvnow();
-    struct timeval  tv_remaining;
+#ifndef CARES_SYMBOL_HIDING
+    ares_timeval_t  tv_now;
+    ares_timeval_t  atv_remaining;
+
+    ares__tvnow(&tv_now);
+#endif
     struct timeval  tv;
     struct timeval *tv_select;
 
@@ -142,21 +308,29 @@ void ProcessWork(ares_channel_t *channel,
     if (tv_select == NULL)
       return;
 
+#ifndef CARES_SYMBOL_HIDING
     if (cancel_ms) {
       unsigned int remaining_ms;
-      ares__timeval_remaining(&tv_remaining,
+      ares__timeval_remaining(&atv_remaining,
                               &tv_now,
                               &tv_cancel);
-      remaining_ms = (unsigned int)((tv_remaining.tv_sec * 1000) + (tv_remaining.tv_usec / 1000));
+
+      remaining_ms = (unsigned int)((atv_remaining.sec * 1000) + (atv_remaining.usec / 1000));
       if (remaining_ms == 0) {
         if (verbose) std::cerr << "Issuing ares_cancel()" << std::endl;
         ares_cancel(channel);
         cancel_ms = 0; /* Disable issuing cancel again */
       } else {
+        struct timeval tv_remaining;
+
+        tv_remaining.tv_sec = atv_remaining.sec;
+        tv_remaining.tv_usec = (int)atv_remaining.usec;
+
         /* Recalculate proper timeout since we also have a cancel to wait on */
         tv_select = ares_timeout(channel, &tv_remaining, &tv);
       }
     }
+#endif
 
     count = select(nfds, &readers, &writers, nullptr, tv_select);
     if (count < 0) {
@@ -176,33 +350,42 @@ void ProcessWork(ares_channel_t *channel,
   }
 }
 
+
 // static
 void LibraryTest::SetAllocFail(int nth) {
+  lock_.lock();
   assert(nth > 0);
   assert(nth <= (int)(8 * sizeof(fails_)));
   fails_ |= (1LL << (nth - 1));
+  lock_.unlock();
 }
 
 // static
 void LibraryTest::SetAllocSizeFail(size_t size) {
+  lock_.lock();
   size_fails_[size]++;
+  lock_.unlock();
 }
 
 // static
 void LibraryTest::ClearFails() {
+  lock_.lock();
   fails_ = 0;
   size_fails_.clear();
+  lock_.unlock();
 }
 
 
 // static
 bool LibraryTest::ShouldAllocFail(size_t size) {
+  lock_.lock();
   bool fail = (fails_ & 0x01);
   fails_ >>= 1;
   if (size_fails_[size] > 0) {
     size_fails_[size]--;
     fail = true;
   }
+  lock_.unlock();
   return fail;
 }
 
@@ -249,6 +432,7 @@ void DefaultChannelModeTest::Process(unsigned int cancel_ms) {
 
 MockServer::MockServer(int family, unsigned short port)
   : udpport_(port), tcpport_(port), qid_(-1) {
+  reply_ = nullptr;
   // Create a TCP socket to receive data on.
   tcp_data_ = NULL;
   tcp_data_len_ = 0;
@@ -340,6 +524,18 @@ MockServer::~MockServer() {
   free(tcp_data_);
 }
 
+static unsigned short getaddrport(struct sockaddr_storage *addr)
+{
+  if (addr->ss_family == AF_INET)
+    return ntohs(((struct sockaddr_in *)(void *)addr)->sin_port);
+  if (addr->ss_family == AF_INET6)
+    return ntohs(((struct sockaddr_in6 *)(void *)addr)->sin6_port);
+
+  /* TCP should use getpeername() to get the port, getting this from recvfrom
+   * won't work */
+  return 0;
+}
+
 void MockServer::ProcessPacket(ares_socket_t fd, struct sockaddr_storage *addr, ares_socklen_t addrlen,
                                byte *data, int len) {
 
@@ -364,8 +560,8 @@ void MockServer::ProcessPacket(ares_socket_t fd, struct sockaddr_storage *addr,
               << ")" << std::endl;
     return;
   }
-  byte* question = data + 12;
-  int qlen = len - 12;
+  byte* question = data + NS_HFIXEDSZ;
+  int qlen = len - NS_HFIXEDSZ;
 
   char *name = nullptr;
   long enclen;
@@ -374,32 +570,39 @@ void MockServer::ProcessPacket(ares_socket_t fd, struct sockaddr_storage *addr,
     std::cerr << "Failed to retrieve name" << std::endl;
     return;
   }
-  qlen -= enclen;
+  if (enclen > qlen) {
+    std::cerr << "(error, encoded name len " << enclen << "bigger than remaining data " << qlen << " bytes)" << std::endl;
+    ares_free_string(name);
+    return;
+  }
+  qlen -= (int)enclen;
   question += enclen;
-  std::string namestr(name);
-  ares_free_string(name);
 
   if (qlen < 4) {
     std::cerr << "Unexpected question size (" << qlen
               << " bytes after name)" << std::endl;
+    ares_free_string(name);
     return;
   }
   if (DNS_QUESTION_CLASS(question) != C_IN) {
     std::cerr << "Unexpected question class (" << DNS_QUESTION_CLASS(question)
               << ")" << std::endl;
+    ares_free_string(name);
     return;
   }
   int rrtype = DNS_QUESTION_TYPE(question);
 
+  std::vector<byte> req(data, data + len);
+  std::string reqstr = PacketToString(req);
   if (verbose) {
-    std::vector<byte> req(data, data + len);
-    std::cerr << "received " << (fd == udpfd_ ? "UDP" : "TCP") << " request " << PacketToString(req)
-              << " on port " << (fd == udpfd_ ? udpport_ : tcpport_) << std::endl;
-    std::cerr << "ProcessRequest(" << qid << ", '" << namestr
+    std::cerr << "received " << (fd == udpfd_ ? "UDP" : "TCP") << " request " << reqstr
+              << " on port " << (fd == udpfd_ ? udpport_ : tcpport_)
+              << ":" << getaddrport(addr) << std::endl;
+    std::cerr << "ProcessRequest(" << qid << ", '" << name
               << "', " << RRTypeToString(rrtype) << ")" << std::endl;
   }
-  ProcessRequest(fd, addr, addrlen, qid, namestr, rrtype);
-
+  ProcessRequest(fd, addr, addrlen, reqstr, qid, name, rrtype);
+  ares_free_string(name);
 }
 
 void MockServer::ProcessFD(ares_socket_t fd) {
@@ -409,7 +612,7 @@ void MockServer::ProcessFD(ares_socket_t fd) {
   }
   if (fd == tcpfd_) {
     ares_socket_t connfd = accept(tcpfd_, NULL, NULL);
-    if (connfd < 0) {
+    if (connfd == ARES_SOCKET_BAD) {
       std::cerr << "Error accepting connection on fd " << fd << std::endl;
     } else {
       connfds_.insert(connfd);
@@ -420,6 +623,7 @@ void MockServer::ProcessFD(ares_socket_t fd) {
   // Activity on a data-bearing file descriptor.
   struct sockaddr_storage addr;
   socklen_t addrlen = sizeof(addr);
+  memset(&addr, 0, sizeof(addr));
   byte buffer[2048];
   ares_ssize_t len = (ares_ssize_t)recvfrom(fd, BYTE_CAST buffer, sizeof(buffer), 0,
                      (struct sockaddr *)&addr, &addrlen);
@@ -466,17 +670,32 @@ std::set<ares_socket_t> MockServer::fds() const {
   return result;
 }
 
-void MockServer::ProcessRequest(ares_socket_t fd, struct sockaddr_storage* addr, ares_socklen_t addrlen,
-                                int qid, const std::string& name, int rrtype) {
-  // Before processing, let gMock know the request is happening.
-  OnRequest(name, rrtype);
+void MockServer::ProcessRequest(ares_socket_t fd, struct sockaddr_storage* addr,
+                                ares_socklen_t addrlen, const std::string &reqstr,
+                                int qid, const char *name, int rrtype) {
 
-  if (reply_.size() == 0) {
+  /* DNS 0x20 will mix case, do case-insensitive matching of name in request */
+  char lower_name[256];
+  arestest_strtolower(lower_name, name, sizeof(lower_name));
+
+  // Before processing, let gMock know the request is happening.
+  OnRequest(lower_name, rrtype);
+
+  // If we are expecting a specific request then check it matches here.
+  if (expected_request_.length() > 0) {
+    ASSERT_EQ(expected_request_, reqstr);
+  }
+
+  if (reply_ != nullptr) {
+    exact_reply_ = reply_->data(name);
+  }
+
+  if (exact_reply_.size() == 0) {
     return;
   }
 
   // Make a local copy of the current pending reply.
-  std::vector<byte> reply = reply_;
+  std::vector<byte> reply = exact_reply_;
 
   if (qid_ >= 0) {
     // Use the explicitly specified query ID.
@@ -487,8 +706,11 @@ void MockServer::ProcessRequest(ares_socket_t fd, struct sockaddr_storage* addr,
     reply[0] = (byte)((qid >> 8) & 0xff);
     reply[1] = (byte)(qid & 0xff);
   }
-  if (verbose) std::cerr << "sending reply " << PacketToString(reply)
-                         << " on port " << ((fd == udpfd_) ? udpport_ : tcpport_) << std::endl;
+  if (verbose) {
+    std::cerr << "sending reply " << PacketToString(reply)
+              << " on port " << ((fd == udpfd_) ? udpport_ : tcpport_)
+              << ":" << getaddrport(addr) << std::endl;
+  }
 
   // Prefix with 2-byte length if TCP.
   if (fd != udpfd_) {
@@ -540,9 +762,9 @@ MockChannelOptsTest::MockChannelOptsTest(int count,
   opts.tcp_port = server_.tcpport();
   optmask |= ARES_OPT_TCP_PORT;
 
-  // If not already overridden, set short-ish timeouts.
   if (!(optmask & (ARES_OPT_TIMEOUTMS|ARES_OPT_TIMEOUT))) {
-    opts.timeout = 1500;
+    // Reduce timeouts significantly to shorten test times.
+    opts.timeout = 250;
     optmask |= ARES_OPT_TIMEOUTMS;
   }
   // If not already overridden, set 3 retries.
@@ -562,6 +784,27 @@ MockChannelOptsTest::MockChannelOptsTest(int count,
     optmask |= ARES_OPT_FLAGS;
   }
 
+  /* Tests expect ndots=1 in general, the system config may not default to this
+   * so we don't want to inherit that. */
+  if (!(optmask & ARES_OPT_NDOTS)) {
+    opts.ndots = 1;
+    optmask |= ARES_OPT_NDOTS;
+  }
+
+  /* Disable the query cache for tests unless explicitly enabled. As of
+   * c-ares 1.31.0, the query cache is enabled by default so we have to set
+   * the option and set the TTL to 0 to effectively disable it. */
+  if (!(optmask & ARES_OPT_QUERY_CACHE)) {
+    opts.qcache_max_ttl = 0;
+    optmask |= ARES_OPT_QUERY_CACHE;
+  }
+
+  /* Enable DNS0x20 by default. Need to also turn on default flag of EDNS */
+  if (!(optmask & ARES_OPT_FLAGS)) {
+    optmask |= ARES_OPT_FLAGS;
+    opts.flags = ARES_FLAG_DNS0x20|ARES_FLAG_EDNS;
+  }
+
   EXPECT_EQ(ARES_SUCCESS, ares_init_options(&channel_, &opts, optmask));
   EXPECT_NE(nullptr, channel_);
 
@@ -631,6 +874,90 @@ void MockChannelOptsTest::Process(unsigned int cancel_ms) {
               cancel_ms);
 }
 
+void MockEventThreadOptsTest::Process(unsigned int cancel_ms) {
+  std::set<ares_socket_t> fds;
+
+#ifndef CARES_SYMBOL_HIDING
+  bool has_cancel_ms = (cancel_ms > 0)?true:false;
+  ares_timeval_t tv_begin;
+  ares_timeval_t tv_cancel;
+#endif
+
+#ifndef CARES_SYMBOL_HIDING
+  ares_timeval_t tv_now;
+  ares_timeval_t atv_remaining;
+
+  if (has_cancel_ms) {
+    ares__tvnow(&tv_begin);
+    memcpy(&tv_cancel, &tv_begin, sizeof(tv_cancel));
+    if (verbose) std::cerr << "ares_cancel will be called after " << cancel_ms << "ms" << std::endl;
+    tv_cancel.sec  += (cancel_ms / 1000);
+    tv_cancel.usec += ((cancel_ms % 1000) * 1000);
+  }
+#else
+  if (cancel_ms) {
+    std::cerr << "library built with symbol hiding, can't test with cancel support" << std::endl;
+    return;
+  }
+#endif
+
+  while (ares_queue_active_queries(channel_)) {
+    int nfds = 0;
+    fd_set readers;
+
+    struct timeval  tv;
+
+    /* c-ares is using its own event thread, so we only need to monitor the
+     * extrafds passed in */
+    FD_ZERO(&readers);
+    fds = MockEventThreadOptsTest::fds();
+    for (ares_socket_t fd : fds) {
+      FD_SET(fd, &readers);
+      if (fd >= (ares_socket_t)nfds) {
+        nfds = (int)fd + 1;
+      }
+    }
+
+    /* We just always wait 20ms then recheck. Not doing any complex signaling. */
+    tv.tv_sec  = 0;
+    tv.tv_usec = 20000;
+
+#ifndef CARES_SYMBOL_HIDING
+    ares__tvnow(&tv_now);
+
+    unsigned int remaining_ms = 0;
+    if (has_cancel_ms) {
+      ares__timeval_remaining(&atv_remaining,
+                              &tv_now,
+                              &tv_cancel);
+      remaining_ms = (unsigned int)((atv_remaining.sec * 1000) + (atv_remaining.usec / 1000));
+      if (remaining_ms == 0) {
+        if (verbose) std::cerr << "Issuing ares_cancel()" << std::endl;
+        ares_cancel(channel_);
+        cancel_ms = 0; /* Disable issuing cancel again */
+        has_cancel_ms = false;
+      }
+    }
+
+    if (has_cancel_ms && remaining_ms < 20) {
+      tv.tv_usec = (int)remaining_ms * 1000;
+    }
+#endif
+
+    if (select(nfds, &readers, nullptr, nullptr, &tv) < 0) {
+      fprintf(stderr, "select() failed, errno %d\n", errno);
+      return;
+    }
+
+    // Let the provided callback process any activity on the extra FD.
+    for (ares_socket_t fd : fds) {
+      if (FD_ISSET(fd, &readers)) {
+        ProcessFD(fd);
+      }
+    }
+  }
+}
+
 std::ostream& operator<<(std::ostream& os, const HostResult& result) {
   os << '{';
   if (result.done_) {
@@ -651,8 +978,13 @@ HostEnt::HostEnt(const struct hostent *hostent) : addrtype_(-1) {
   if (!hostent)
     return;
 
-  if (hostent->h_name)
-    name_ = hostent->h_name;
+  if (hostent->h_name) {
+    // DNS 0x20 may mix case, output as all lower for checks as the mixed case
+    // is really more of an internal thing
+    char lowername[256];
+    arestest_strtolower(lowername, hostent->h_name, sizeof(lowername));
+    name_ = lowername;
+  }
 
   if (hostent->h_aliases) {
     char** palias = hostent->h_aliases;
@@ -736,7 +1068,10 @@ std::ostream& operator<<(std::ostream& os, const AddrInfo& ai) {
     if(next_cname->name) {
       os << next_cname->name;
     }
-    if((next_cname = next_cname->next))
+
+    next_cname = next_cname->next;
+
+    if (next_cname != NULL)
       os << ", ";
     else
       os << " ";
@@ -765,7 +1100,8 @@ std::ostream& operator<<(std::ostream& os, const AddrInfo& ai) {
       os << ":" << port;
     }
     os << "]";
-    if((next = next->ai_next))
+    next = next->ai_next;
+    if (next != NULL)
       os << ", ";
   }
   os << '}';
@@ -806,6 +1142,23 @@ void SearchCallback(void *data, int status, int timeouts,
   if (verbose) std::cerr << "SearchCallback(" << *result << ")" << std::endl;
 }
 
+void SearchCallbackDnsRec(void *data, ares_status_t status, size_t timeouts,
+                          const ares_dns_record_t *dnsrec) {
+  EXPECT_NE(nullptr, data);
+  SearchResult* result = reinterpret_cast<SearchResult*>(data);
+  unsigned char *abuf = NULL;
+  size_t alen = 0;
+  result->done_ = true;
+  result->status_ = (int)status;
+  result->timeouts_ = (int)timeouts;
+  if (dnsrec != NULL) {
+    ares_dns_write(dnsrec, &abuf, &alen);
+  }
+  result->data_.assign(abuf, abuf + alen);
+  ares_free_string(abuf);
+  if (verbose) std::cerr << "SearchCallbackDnsRec(" << *result << ")" << std::endl;
+}
+
 std::ostream& operator<<(std::ostream& os, const NameInfoResult& result) {
   os << '{';
   if (result.done_) {
