@@ -2,8 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2021 The Fluent Bit Authors
- *  Copyright (C) 2015-2018 Treasure Data Inc.
+ *  Copyright (C) 2015-2024 The Fluent Bit Authors
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -27,6 +26,7 @@
 #include <fluent-bit/flb_record_accessor.h>
 
 /* rule types */
+#define GREP_NO_RULE  0
 #define GREP_REGEX    1
 #define GREP_EXCLUDE  2
 
@@ -34,8 +34,15 @@
 #define GREP_RET_KEEP     0
 #define GREP_RET_EXCLUDE  1
 
+enum _logical_op{
+    GREP_LOGICAL_OP_LEGACY,
+    GREP_LOGICAL_OP_OR,
+    GREP_LOGICAL_OP_AND
+} logical_op;
+
 struct grep_ctx {
     struct mk_list rules;
+    int logical_op;
     struct flb_filter_instance *ins;
 };
 
