@@ -56,6 +56,7 @@ static int cb_wasm_filter(const void *data, size_t bytes,
     size_t json_size;
     int root_type;
     struct flb_wasm *wasm = NULL;
+    size_t buf_size;
 
     struct flb_filter_wasm *ctx = filter_context;
     struct flb_log_event_encoder log_encoder;
@@ -99,23 +100,44 @@ static int cb_wasm_filter(const void *data, size_t bytes,
         off = log_decoder.offset;
         alloc_size = (off - last_off) + 128; /* JSON is larger than msgpack */
         last_off = off;
+        switch(ctx->event_format) {
+        case FLB_FILTER_WASM_FMT_JSON:
+            /* Encode as JSON from msgpack */
+            buf = flb_msgpack_to_json_str(alloc_size, log_event.body);
 
-        /* Encode as JSON from msgpack */
-        buf = flb_msgpack_to_json_str(alloc_size, log_event.body);
+            if (buf) {
+                /* Execute WASM program */
+                ret_val = flb_wasm_call_function_format_json(wasm, ctx->wasm_function_name,
+                                                             tag, tag_len,
+                                                             log_event.timestamp,
+                                                             buf, strlen(buf));
+
+                flb_free(buf);
+            }
+            else {
+                flb_plg_error(ctx->ins, "encode as JSON from msgpack is failed");
+
+                goto on_error;
+            }
+            break;
+        case FLB_FILTER_WASM_FMT_MSGPACK:
+            ret = flb_wasm_format_msgpack_mode(tag, tag_len,
+                                               &log_event,
+                                               (void **)&buf, &buf_size);
+            if (ret < 0) {
+                flb_plg_error(ctx->ins, "format msgpack is failed");
+
+                goto on_error;
+            }
 
-        if (buf) {
             /* Execute WASM program */
-            ret_val = flb_wasm_call_function_format_json(wasm, ctx->wasm_function_name,
-                                                         tag, tag_len,
-                                                         log_event.timestamp,
-                                                         buf, strlen(buf));
+            ret_val = flb_wasm_call_function_format_msgpack(wasm, ctx->wasm_function_name,
+                                                            tag, tag_len,
+                                                            log_event.timestamp,
+                                                            buf, buf_size);
 
             flb_free(buf);
-        }
-        else {
-            flb_plg_error(ctx->ins, "encode as JSON from msgpack is failed");
-
-            goto on_error;
+            break;
         }
 
         if (ret_val == NULL) { /* Skip record */
@@ -123,8 +145,8 @@ static int cb_wasm_filter(const void *data, size_t bytes,
             continue;
         }
 
-        
-        if (strlen(ret_val) == 0) { /* Skip record */
+        if (ctx->event_format == FLB_FILTER_WASM_FMT_JSON &&
+            strlen(ret_val) == 0) { /* Skip record */
             flb_plg_debug(ctx->ins, "WASM function returned empty string. Skip.");
             flb_free(ret_val);
             continue;
@@ -144,16 +166,38 @@ static int cb_wasm_filter(const void *data, size_t bytes,
         }
 
         if (ret == FLB_EVENT_ENCODER_SUCCESS) {
-            /* Convert JSON payload to msgpack */
-            ret = flb_pack_json(ret_val, strlen(ret_val),
-                                &json_buf, &json_size, &root_type, NULL);
+            switch(ctx->event_format) {
+            case FLB_FILTER_WASM_FMT_JSON:
+                /* Convert JSON payload to msgpack */
+                ret = flb_pack_json(ret_val, strlen(ret_val),
+                                    &json_buf, &json_size, &root_type, NULL);
 
-            if (ret == 0 && root_type == JSMN_OBJECT) {
-                /* JSON found, pack it msgpack representation */
+                if (ret == 0 && root_type == JSMN_OBJECT) {
+                    /* JSON found, pack it msgpack representation */
+                    ret = flb_log_event_encoder_set_body_from_raw_msgpack(
+                            &log_encoder,
+                            json_buf,
+                            json_size);
+
+                    if (ret == FLB_EVENT_ENCODER_SUCCESS) {
+                        ret = flb_log_event_encoder_commit_record(&log_encoder);
+                    }
+                    else {
+                        flb_log_event_encoder_rollback_record(&log_encoder);
+                    }
+                }
+                else {
+                    flb_plg_error(ctx->ins, "invalid JSON format. ret: %d, buf: %s", ret, ret_val);
+
+                    flb_log_event_encoder_rollback_record(&log_encoder);
+                }
+                break;
+            case FLB_FILTER_WASM_FMT_MSGPACK:
+                /* msgpack found, pack it msgpack representation */
                 ret = flb_log_event_encoder_set_body_from_raw_msgpack(
                         &log_encoder,
-                        json_buf,
-                        json_size);
+                        ret_val,
+                        strlen(ret_val));
 
                 if (ret == FLB_EVENT_ENCODER_SUCCESS) {
                     ret = flb_log_event_encoder_commit_record(&log_encoder);
@@ -161,11 +205,8 @@ static int cb_wasm_filter(const void *data, size_t bytes,
                 else {
                     flb_log_event_encoder_rollback_record(&log_encoder);
                 }
-            }
-            else {
-                flb_plg_error(ctx->ins, "invalid JSON format. ret: %d, buf: %s", ret, ret_val);
 
-                flb_log_event_encoder_rollback_record(&log_encoder);
+                break;
             }
         }
         else {
@@ -177,9 +218,11 @@ static int cb_wasm_filter(const void *data, size_t bytes,
             flb_free(ret_val);
         }
 
-        /* release 'json_buf' if it was allocated */
-        if (json_buf != NULL) {
-            flb_free(json_buf);
+        if (ctx->event_format == FLB_FILTER_WASM_FMT_JSON) {
+            /* release 'json_buf' if it was allocated */
+            if (json_buf != NULL) {
+                flb_free(json_buf);
+            }
         }
     }
 
@@ -288,6 +331,8 @@ static int cb_wasm_init(struct flb_filter_instance *f_ins,
 {
     struct flb_filter_wasm *ctx = NULL;
     int ret = -1;
+    const char *tmp;
+    int event_format;
 
     /* Allocate space for the configuration */
     ctx = flb_calloc(1, sizeof(struct flb_filter_wasm));
@@ -301,6 +346,22 @@ static int cb_wasm_init(struct flb_filter_instance *f_ins,
         goto init_error;
     }
 
+    tmp = flb_filter_get_property("event_format", f_ins);
+    if (tmp) {
+        if (strcasecmp(tmp, FLB_FMT_STR_JSON) == 0) {
+            event_format = FLB_FILTER_WASM_FMT_JSON;
+        }
+        else if (strcasecmp(tmp, FLB_FMT_STR_MSGPACK) == 0) {
+            event_format = FLB_FILTER_WASM_FMT_MSGPACK;
+        } else {
+            flb_error("[filter_wasm] unknown format: %s", tmp);
+            goto init_error;
+        }
+        ctx->event_format = event_format;
+    } else {
+        ctx->event_format = FLB_FILTER_WASM_FMT_JSON;
+    }
+
     flb_wasm_init(config);
 
     /* Set context */
@@ -323,6 +384,11 @@ static int cb_wasm_exit(void *data, struct flb_config *config)
 }
 
 static struct flb_config_map config_map[] = {
+    {
+     FLB_CONFIG_MAP_STR, "event_format", NULL,
+     0, FLB_FALSE, 0,
+     "Sepecify the ingesting event format for wasm program"
+    },
     {
      FLB_CONFIG_MAP_STR, "wasm_path", NULL,
      0, FLB_TRUE, offsetof(struct flb_filter_wasm, wasm_path),
