@@ -2,8 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2021 The Fluent Bit Authors
- *  Copyright (C) 2015-2018 Treasure Data Inc.
+ *  Copyright (C) 2015-2024 The Fluent Bit Authors
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -22,6 +21,7 @@
 #define FLB_IN_COLLECTD_H
 
 #include <fluent-bit/flb_input_plugin.h>
+#include <fluent-bit/flb_log_event_encoder.h>
 
 struct flb_in_collectd_config {
     char *buf;
@@ -35,7 +35,9 @@ struct flb_in_collectd_config {
     flb_sockfd_t server_fd;
     flb_pipefd_t coll_fd;
 
+    flb_sds_t types_db;
     struct mk_list *tdb;
+    struct flb_log_event_encoder log_encoder;
 
     /* Plugin input instance */
     struct flb_input_instance *ins;
