@@ -84,7 +84,7 @@ static int send_response(struct splunk_conn *conn, int http_status, char *messag
     }
     else if (http_status == 400) {
         flb_sds_printf(&out,
-                       "HTTP/1.1 400 Forbidden\r\n"
+                       "HTTP/1.1 400 Bad Request\r\n"
                        "Server: Fluent Bit v%s\r\n"
                        "Content-Length: %i\r\n\r\n%s",
                        FLB_VERSION_STR,
@@ -241,7 +241,8 @@ static int process_raw_payload_pack(struct flb_splunk *ctx, flb_sds_t tag, char
                 ret = flb_log_event_encoder_append_metadata_values(
                     &ctx->log_encoder,
                     FLB_LOG_EVENT_CSTRING_VALUE("hec_token"),
-                    FLB_LOG_EVENT_CSTRING_VALUE(ctx->ingested_auth_header));
+                    FLB_LOG_EVENT_STRING_VALUE(ctx->ingested_auth_header,
+                                               ctx->ingested_auth_header_len));
             }
         }
     }
@@ -251,7 +252,8 @@ static int process_raw_payload_pack(struct flb_splunk *ctx, flb_sds_t tag, char
                 ret = flb_log_event_encoder_append_body_values(
                     &ctx->log_encoder,
                     FLB_LOG_EVENT_CSTRING_VALUE(ctx->store_token_key),
-                    FLB_LOG_EVENT_CSTRING_VALUE(ctx->ingested_auth_header),
+                    FLB_LOG_EVENT_STRING_VALUE(ctx->ingested_auth_header,
+                                               ctx->ingested_auth_header_len),
                     FLB_LOG_EVENT_CSTRING_VALUE("log"),
                     FLB_LOG_EVENT_STRING_VALUE(buf, size));
 
@@ -315,7 +317,8 @@ static void process_flb_log_append(struct flb_splunk *ctx, msgpack_object *recor
                 ret = flb_log_event_encoder_append_metadata_values(
                     &ctx->log_encoder,
                     FLB_LOG_EVENT_CSTRING_VALUE("hec_token"),
-                    FLB_LOG_EVENT_CSTRING_VALUE(ctx->ingested_auth_header));
+                    FLB_LOG_EVENT_STRING_VALUE(ctx->ingested_auth_header,
+                                               ctx->ingested_auth_header_len));
             }
         }
     }
@@ -334,7 +337,8 @@ static void process_flb_log_append(struct flb_splunk *ctx, msgpack_object *recor
                 ret = flb_log_event_encoder_append_body_values(
                     &ctx->log_encoder,
                     FLB_LOG_EVENT_CSTRING_VALUE(ctx->store_token_key),
-                    FLB_LOG_EVENT_CSTRING_VALUE(ctx->ingested_auth_header));
+                    FLB_LOG_EVENT_STRING_VALUE(ctx->ingested_auth_header,
+                                               ctx->ingested_auth_header_len));
             }
         }
         else {
@@ -481,31 +485,47 @@ static ssize_t parse_hec_payload_json(struct flb_splunk *ctx, flb_sds_t tag,
 
 static int validate_auth_header(struct flb_splunk *ctx, struct mk_http_request *request)
 {
+    int ret = 0;
     struct mk_list *head;
     struct mk_http_header *auth_header = NULL;
     struct flb_splunk_tokens *splunk_token;
+    flb_sds_t authorization = NULL;
 
     if (mk_list_size(&ctx->auth_tokens) == 0) {
         return SPLUNK_AUTH_UNAUTH;
     }
 
     auth_header = mk_http_header_get(MK_HEADER_AUTHORIZATION, request, NULL, 0);
-
     if (auth_header == NULL) {
         return SPLUNK_AUTH_MISSING_CRED;
     }
 
-    if (auth_header != NULL && auth_header->val.len > 0) {
+    authorization = flb_sds_create_len(auth_header->val.data, auth_header->val.len);
+    if (authorization == NULL) {
+        return SPLUNK_AUTH_MISSING_CRED;
+    }
+
+    if (authorization != NULL && auth_header->val.len > 0) {
         mk_list_foreach(head, &ctx->auth_tokens) {
             splunk_token = mk_list_entry(head, struct flb_splunk_tokens, _head);
+            if (flb_sds_len(authorization) != splunk_token->length) {
+                ret = SPLUNK_AUTH_UNAUTHORIZED;
+                continue;
+            }
+
             if (strncmp(splunk_token->header,
-                        auth_header->val.data,
-                        strlen(splunk_token->header)) == 0) {
+                        authorization,
+                        splunk_token->length) == 0) {
+                flb_sds_destroy(authorization);
+
                 return SPLUNK_AUTH_SUCCESS;
             }
         }
 
-        return SPLUNK_AUTH_UNAUTHORIZED;
+        ret = SPLUNK_AUTH_UNAUTHORIZED;
+        flb_sds_destroy(authorization);
+
+        return ret;
     }
     else {
         return SPLUNK_AUTH_MISSING_CRED;
@@ -555,8 +575,7 @@ static int process_hec_payload(struct flb_splunk *ctx, struct splunk_conn *conn,
 
     header = &session->parser.headers[MK_HEADER_CONTENT_TYPE];
     if (header->key.data == NULL) {
-        send_response(conn, 400, "error: header 'Content-Type' is not set\n");
-        return -1;
+        flb_plg_debug(ctx->ins, "header 'Content-Type' is not set");
     }
 
     if (header->val.len == 16 &&
@@ -568,20 +587,21 @@ static int process_hec_payload(struct flb_splunk *ctx, struct splunk_conn *conn,
         type = HTTP_CONTENT_TEXT;
     }
     else {
-        /* Not neccesary to specify content-type for Splunk HEC. */
+        /* Not necessary to specify content-type for Splunk HEC. */
         flb_plg_debug(ctx->ins, "Mark as unknown type for ingested payloads");
         type = HTTP_CONTENT_UNKNOWN;
     }
 
     if (request->data.len <= 0) {
         send_response(conn, 400, "error: no payload found\n");
-        return -1;
+        return -2;
     }
 
     header_auth = &session->parser.headers[MK_HEADER_AUTHORIZATION];
     if (header_auth->key.data != NULL) {
         if (strncasecmp(header_auth->val.data, "Splunk ", 7) == 0) {
             ctx->ingested_auth_header = header_auth->val.data;
+            ctx->ingested_auth_header_len = header_auth->val.len;
         }
     }
 
@@ -634,7 +654,7 @@ static int process_hec_raw_payload(struct flb_splunk *ctx, struct splunk_conn *c
     }
     else if (header->val.len != 10 ||
              strncasecmp(header->val.data, "text/plain", 10) != 0) {
-        /* Not neccesary to specify content-type for Splunk HEC. */
+        /* Not necessary to specify content-type for Splunk HEC. */
         flb_plg_debug(ctx->ins, "Mark as unknown type for ingested payloads");
     }
 
@@ -647,6 +667,7 @@ static int process_hec_raw_payload(struct flb_splunk *ctx, struct splunk_conn *c
     if (header_auth->key.data != NULL) {
         if (strncasecmp(header_auth->val.data, "Splunk ", 7) == 0) {
             ctx->ingested_auth_header = header_auth->val.data;
+            ctx->ingested_auth_header_len = header_auth->val.len;
         }
     }
 
@@ -776,7 +797,7 @@ int splunk_prot_handle(struct flb_splunk *ctx, struct splunk_conn *conn,
     }
 
     if (request->method == MK_METHOD_GET) {
-        /* Handle health minotoring of splunk hec endpoint for load balancers */
+        /* Handle health monitoring of splunk hec endpoint for load balancers */
         if (strcasecmp(uri, "/services/collector/health") == 0) {
             send_json_message_response(conn, 200, "{\"text\":\"Success\",\"code\":200}");
         }
@@ -794,7 +815,7 @@ int splunk_prot_handle(struct flb_splunk *ctx, struct splunk_conn *conn,
      * authentication if provided splunk_token */
     ret = validate_auth_header(ctx, request);
     if (ret < 0){
-        send_response(conn, 401, "error: unauthroized\n");
+        send_response(conn, 401, "error: unauthorized\n");
         if (ret == SPLUNK_AUTH_MISSING_CRED) {
             flb_plg_warn(ctx->ins, "missing credentials in request headers");
         }
@@ -824,6 +845,13 @@ int splunk_prot_handle(struct flb_splunk *ctx, struct splunk_conn *conn,
                  strcasecmp(uri, "/services/collector") == 0) {
             ret = process_hec_payload(ctx, conn, tag, session, request);
 
+            if (ret == -2) {
+                flb_sds_destroy(tag);
+                mk_mem_free(uri);
+
+                return -1;
+            }
+
             if (!ret) {
                 send_json_message_response(conn, 400, "{\"text\":\"Invalid data format\",\"code\":6}");
             }
@@ -889,7 +917,7 @@ static int send_response_ng(struct flb_http_response *response,
         flb_http_response_set_message(response, "No Content");
     }
     else if (http_status == 400) {
-        flb_http_response_set_message(response, "Forbidden");
+        flb_http_response_set_message(response, "Bad Request");
     }
 
     if (message != NULL) {
@@ -919,7 +947,7 @@ static int send_json_message_response_ng(struct flb_http_response *response,
         flb_http_response_set_message(response, "No Content");
     }
     else if (http_status == 400) {
-        flb_http_response_set_message(response, "Forbidden");
+        flb_http_response_set_message(response, "Bad Request");
     }
 
     flb_http_response_set_header(response, 
@@ -957,9 +985,13 @@ static int validate_auth_header_ng(struct flb_splunk *ctx, struct flb_http_reque
     if (auth_header != NULL && strlen(auth_header) > 0) {
         mk_list_foreach_safe(head, tmp, &ctx->auth_tokens) {
             splunk_token = mk_list_entry(head, struct flb_splunk_tokens, _head);
+            if (strlen(auth_header) != splunk_token->length) {
+                return SPLUNK_AUTH_UNAUTHORIZED;
+            }
+
             if (strncmp(splunk_token->header,
                         auth_header,
-                        strlen(splunk_token->header)) == 0) {
+                        splunk_token->length) == 0) {
                 return SPLUNK_AUTH_SUCCESS;
             }
         }
@@ -993,7 +1025,7 @@ static int process_hec_payload_ng(struct flb_http_request *request,
             type = HTTP_CONTENT_TEXT;
         }
         else {
-            /* Not neccesary to specify content-type for Splunk HEC. */
+            /* Not necessary to specify content-type for Splunk HEC. */
             flb_plg_debug(ctx->ins, "Mark as unknown type for ingested payloads");
         }
     }
@@ -1002,6 +1034,7 @@ static int process_hec_payload_ng(struct flb_http_request *request,
     if (ret != 0 && size > 0) {
         if (strncasecmp(auth_header, "Splunk ", 7) == 0) {
             ctx->ingested_auth_header = auth_header;
+            ctx->ingested_auth_header_len = strlen(auth_header);
         }
     }
 
@@ -1029,7 +1062,7 @@ static int process_hec_raw_payload_ng(struct flb_http_request *request,
         return -1;
     }
     else if (strcasecmp(request->content_type, "text/plain") != 0) {
-        /* Not neccesary to specify content-type for Splunk HEC. */
+        /* Not necessary to specify content-type for Splunk HEC. */
         flb_plg_debug(ctx->ins, "Mark as unknown type for ingested payloads");
     }
 
@@ -1037,6 +1070,7 @@ static int process_hec_raw_payload_ng(struct flb_http_request *request,
     if (ret != 0 && size > 0) {
         if (strncasecmp(auth_header, "Splunk ", 7) == 0) {
             ctx->ingested_auth_header = auth_header;
+            ctx->ingested_auth_header_len = strlen(auth_header);
         }
     }
 
@@ -1072,7 +1106,7 @@ int splunk_prot_handle_ng(struct flb_http_request *request,
     }
 
     if (request->method == HTTP_METHOD_GET) {
-        /* Handle health minotoring of splunk hec endpoint for load balancers */
+        /* Handle health monitoring of splunk hec endpoint for load balancers */
         if (strcasecmp(request->path, "/services/collector/health") == 0) {
             send_json_message_response_ng(response, 200, "{\"text\":\"Success\",\"code\":200}");
         }
@@ -1134,6 +1168,8 @@ int splunk_prot_handle_ng(struct flb_http_request *request,
 
         if (ret != 0) {
             send_json_message_response_ng(response, 400, "{\"text\":\"Invalid data format\",\"code\":6}");
+
+            ret = -1;
         }
         else {
             send_json_message_response_ng(response, 200, "{\"text\":\"Success\",\"code\":0}");
