@@ -2,8 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2021 The Fluent Bit Authors
- *  Copyright (C) 2015-2018 Treasure Data Inc.
+ *  Copyright (C) 2015-2024 The Fluent Bit Authors
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -30,10 +29,8 @@ int flb_tail_dmode_process_content(time_t now,
                                    char* line, size_t line_len,
                                    char **repl_line, size_t *repl_line_len,
                                    struct flb_tail_file *file,
-                                   struct flb_tail_config *ctx,
-                                   msgpack_sbuffer *mp_sbuf, msgpack_packer *mp_pck);
-void flb_tail_dmode_flush(msgpack_sbuffer *mp_sbuf, msgpack_packer *mp_pck,
-                          struct flb_tail_file *file, struct flb_tail_config *ctx);
+                                   struct flb_tail_config *ctx);
+void flb_tail_dmode_flush(struct flb_tail_file *file, struct flb_tail_config *ctx);
 int flb_tail_dmode_pending_flush(struct flb_input_instance *ins,
                                  struct flb_config *config, void *context);
 int flb_tail_dmode_pending_flush_all(struct flb_tail_config *ctx);
