@@ -2,8 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2021 The Fluent Bit Authors
- *  Copyright (C) 2015-2018 Treasure Data Inc.
+ *  Copyright (C) 2015-2024 The Fluent Bit Authors
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -22,8 +21,9 @@
 #define FLB_IN_TCP_CONN_H
 
 #include <fluent-bit/flb_pack.h>
+#include <fluent-bit/flb_connection.h>
 
-#define FLB_IN_TCP_CHUNK 32768
+#define FLB_IN_TCP_CHUNK "32768"
 
 enum {
     TCP_NEW        = 1,  /* it's a new connection                */
@@ -37,8 +37,6 @@ struct tcp_conn_stream {
 
 /* Respresents a connection */
 struct tcp_conn {
-    struct mk_event event;            /* Built-in event data for mk_events */
-    int fd;                           /* Socket file descriptor            */
     int status;                       /* Connection status                 */
 
     /* Buffer */
@@ -50,11 +48,12 @@ struct tcp_conn {
     struct flb_input_instance *ins;   /* Parent plugin instance            */
     struct flb_in_tcp_config *ctx;    /* Plugin configuration context      */
     struct flb_pack_state pack_state; /* Internal JSON parser              */
+    struct flb_connection *connection;
 
     struct mk_list _head;
 };
 
-struct tcp_conn *tcp_conn_add(int fd, struct flb_in_tcp_config *ctx);
+struct tcp_conn *tcp_conn_add(struct flb_connection *connection, struct flb_in_tcp_config *ctx);
 int tcp_conn_del(struct tcp_conn *conn);
 
 #endif
