@@ -2,8 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2021 The Fluent Bit Authors
- *  Copyright (C) 2015-2018 Treasure Data Inc.
+ *  Copyright (C) 2015-2024 The Fluent Bit Authors
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -26,8 +25,10 @@
 #include <fluent-bit/flb_time.h>
 #include <fluent-bit/flb_signv4.h>
 #include <fluent-bit/flb_aws_credentials.h>
+#include <fluent-bit/flb_gzip.h>
 #include <fluent-bit/flb_record_accessor.h>
 #include <fluent-bit/flb_ra_key.h>
+#include <fluent-bit/flb_log_event_decoder.h>
 #include <msgpack.h>
 
 #include <time.h>
@@ -63,8 +64,8 @@ static flb_sds_t add_aws_auth(struct flb_http_client *c,
     flb_http_add_header(c, "User-Agent", 10, "aws-fluent-bit-plugin", 21);
 
     signature = flb_signv4_do(c, FLB_TRUE, FLB_TRUE, time(NULL),
-                              ctx->aws_region, "es",
-                              0,
+                              ctx->aws_region, ctx->aws_service_name,
+                              S3_MODE_SIGNED_PAYLOAD, ctx->aws_unsigned_headers,
                               ctx->aws_provider);
     if (!signature) {
         flb_plg_error(ctx->ins, "could not sign request with sigv4");
@@ -232,6 +233,45 @@ static flb_sds_t es_get_id_value(struct flb_elasticsearch *ctx,
     return tmp_str;
 }
 
+static int compose_index_header(struct flb_elasticsearch *ctx,
+                                int es_index_custom_len,
+                                char *logstash_index, size_t logstash_index_size,
+                                char *separator_str,
+                                struct tm *tm)
+{
+    int ret;
+    int len;
+    char *p;
+    size_t s;
+
+    /* Compose Index header */
+    if (es_index_custom_len > 0) {
+        p = logstash_index + es_index_custom_len;
+    } else {
+        p = logstash_index + flb_sds_len(ctx->logstash_prefix);
+    }
+    len = p - logstash_index;
+    ret = snprintf(p, logstash_index_size - len, "%s",
+                   separator_str);
+    if (ret > logstash_index_size - len) {
+        /* exceed limit */
+        return -1;
+    }
+    p += strlen(separator_str);
+    len += strlen(separator_str);
+
+    s = strftime(p, logstash_index_size - len,
+                 ctx->logstash_dateformat, tm);
+    if (s==0) {
+        /* exceed limit */
+        return -1;
+    }
+    p += s;
+    *p++ = '\0';
+
+    return 0;
+}
+
 /*
  * Convert the internal Fluent Bit data representation to the required
  * one by Elasticsearch.
@@ -242,6 +282,7 @@ static int elasticsearch_format(struct flb_config *config,
                                 struct flb_input_instance *ins,
                                 void *plugin_context,
                                 void *flush_ctx,
+                                int event_type,
                                 const char *tag, int tag_len,
                                 const void *data, size_t bytes,
                                 void **out_data, size_t *out_size)
@@ -253,19 +294,20 @@ static int elasticsearch_format(struct flb_config *config,
     size_t s = 0;
     size_t off = 0;
     size_t off_prev = 0;
-    char *p;
     char *es_index;
     char logstash_index[256];
     char time_formatted[256];
     char index_formatted[256];
     char es_uuid[37];
     flb_sds_t out_buf;
+    size_t out_buf_len = 0;
+    flb_sds_t tmp_buf;
     flb_sds_t id_key_str = NULL;
-    msgpack_unpacked result;
-    msgpack_object root;
+    // msgpack_unpacked result;
+    // msgpack_object root;
     msgpack_object map;
-    msgpack_object *obj;
-    char j_index[ES_BULK_HEADER];
+    // msgpack_object *obj;
+    flb_sds_t j_index;
     struct es_bulk *bulk;
     struct tm tm;
     struct flb_time tms;
@@ -274,47 +316,37 @@ static int elasticsearch_format(struct flb_config *config,
     uint16_t hash[8];
     int es_index_custom_len;
     struct flb_elasticsearch *ctx = plugin_context;
+    struct flb_log_event_decoder log_decoder;
+    struct flb_log_event log_event;
 
-    /* Iterate the original buffer and perform adjustments */
-    msgpack_unpacked_init(&result);
-
-    /* Perform some format validation */
-    ret = msgpack_unpack_next(&result, data, bytes, &off);
-    if (ret != MSGPACK_UNPACK_SUCCESS) {
-        msgpack_unpacked_destroy(&result);
+    j_index = flb_sds_create_size(ES_BULK_HEADER);
+    if (j_index == NULL) {
+        flb_errno();
         return -1;
     }
 
-    /* We 'should' get an array */
-    if (result.data.type != MSGPACK_OBJECT_ARRAY) {
-        /*
-         * If we got a different format, we assume the caller knows what he is
-         * doing, we just duplicate the content in a new buffer and cleanup.
-         */
-        msgpack_unpacked_destroy(&result);
-        return -1;
-    }
+    ret = flb_log_event_decoder_init(&log_decoder, (char *) data, bytes);
+
+    if (ret != FLB_EVENT_DECODER_SUCCESS) {
+        flb_plg_error(ctx->ins,
+                      "Log event decoder initialization error : %d", ret);
+        flb_sds_destroy(j_index);
 
-    root = result.data;
-    if (root.via.array.size == 0) {
         return -1;
     }
 
     /* Create the bulk composer */
     bulk = es_bulk_create(bytes);
     if (!bulk) {
+        flb_log_event_decoder_destroy(&log_decoder);
+        flb_sds_destroy(j_index);
         return -1;
     }
 
-    off = 0;
-
-    msgpack_unpacked_destroy(&result);
-    msgpack_unpacked_init(&result);
-
     /* Copy logstash prefix if logstash format is enabled */
     if (ctx->logstash_format == FLB_TRUE) {
-        memcpy(logstash_index, ctx->logstash_prefix, flb_sds_len(ctx->logstash_prefix));
-        logstash_index[flb_sds_len(ctx->logstash_prefix)] = '\0';
+        strncpy(logstash_index, ctx->logstash_prefix, sizeof(logstash_index));
+        logstash_index[sizeof(logstash_index) - 1] = '\0';
     }
 
     /*
@@ -331,16 +363,18 @@ static int elasticsearch_format(struct flb_config *config,
                  ctx->index, &tm);
         es_index = index_formatted;
         if (ctx->suppress_type_name) {
-            index_len = snprintf(j_index,
-                                 ES_BULK_HEADER,
-                                 ES_BULK_INDEX_FMT_WITHOUT_TYPE,
-                                 es_index);
+            index_len = flb_sds_snprintf(&j_index,
+                                         flb_sds_alloc(j_index),
+                                         ES_BULK_INDEX_FMT_WITHOUT_TYPE,
+                                         ctx->es_action,
+                                         es_index);
         }
         else {
-            index_len = snprintf(j_index,
-                                 ES_BULK_HEADER,
-                                 ES_BULK_INDEX_FMT,
-                                 es_index, ctx->type);
+            index_len = flb_sds_snprintf(&j_index,
+                                         flb_sds_alloc(j_index),
+                                         ES_BULK_INDEX_FMT,
+                                         ctx->es_action,
+                                         es_index, ctx->type);
         }
     }
 
@@ -355,23 +389,16 @@ static int elasticsearch_format(struct flb_config *config,
     }
 
     /* Iterate each record and do further formatting */
-    while (msgpack_unpack_next(&result, data, bytes, &off) == MSGPACK_UNPACK_SUCCESS) {
-        if (result.data.type != MSGPACK_OBJECT_ARRAY) {
-            continue;
-        }
-
-        /* Each array must have two entries: time and record */
-        root = result.data;
-        if (root.via.array.size != 2) {
-            continue;
-        }
+    while ((ret = flb_log_event_decoder_next(
+                    &log_decoder,
+                    &log_event)) == FLB_EVENT_DECODER_SUCCESS) {
 
         /* Only pop time from record if current_time_index is disabled */
         if (ctx->current_time_index == FLB_FALSE) {
-            flb_time_pop_from_msgpack(&tms, &result, &obj);
+            flb_time_copy(&tms, &log_event.timestamp);
         }
 
-        map   = root.via.array.ptr[1];
+        map   = *log_event.body;
         map_size = map.via.map.size;
 
         es_index_custom_len = 0;
@@ -427,32 +454,31 @@ static int elasticsearch_format(struct flb_config *config,
 
         es_index = ctx->index;
         if (ctx->logstash_format == FLB_TRUE) {
-            /* Compose Index header */
-            if (es_index_custom_len > 0) {
-                p = logstash_index + es_index_custom_len;
-            } else {
-                p = logstash_index + flb_sds_len(ctx->logstash_prefix);
+            ret = compose_index_header(ctx, es_index_custom_len,
+                                       &logstash_index[0], sizeof(logstash_index),
+                                       ctx->logstash_prefix_separator, &tm);
+            if (ret < 0) {
+                /* retry with default separator */
+                compose_index_header(ctx, es_index_custom_len,
+                                     &logstash_index[0], sizeof(logstash_index),
+                                     "-", &tm);
             }
-            *p++ = '-';
 
-            len = p - logstash_index;
-            s = strftime(p, sizeof(logstash_index) - len - 1,
-                         ctx->logstash_dateformat, &tm);
-            p += s;
-            *p++ = '\0';
             es_index = logstash_index;
             if (ctx->generate_id == FLB_FALSE) {
                 if (ctx->suppress_type_name) {
-                    index_len = snprintf(j_index,
-                                         ES_BULK_HEADER,
-                                         ES_BULK_INDEX_FMT_WITHOUT_TYPE,
-                                         es_index);
+                    index_len = flb_sds_snprintf(&j_index,
+                                                 flb_sds_alloc(j_index),
+                                                 ES_BULK_INDEX_FMT_WITHOUT_TYPE,
+                                                 ctx->es_action,
+                                                 es_index);
                 }
                 else {
-                    index_len = snprintf(j_index,
-                                         ES_BULK_HEADER,
-                                         ES_BULK_INDEX_FMT,
-                                         es_index, ctx->type);
+                    index_len = flb_sds_snprintf(&j_index,
+                                                 flb_sds_alloc(j_index),
+                                                 ES_BULK_INDEX_FMT,
+                                                 ctx->es_action,
+                                                 es_index, ctx->type);
                 }
             }
         }
@@ -480,9 +506,10 @@ static int elasticsearch_format(struct flb_config *config,
          */
         ret = es_pack_map_content(&tmp_pck, map, ctx);
         if (ret == -1) {
-            msgpack_unpacked_destroy(&result);
+            flb_log_event_decoder_destroy(&log_decoder);
             msgpack_sbuffer_destroy(&tmp_sbuf);
             es_bulk_destroy(bulk);
+            flb_sds_destroy(j_index);
             return -1;
         }
 
@@ -493,32 +520,36 @@ static int elasticsearch_format(struct flb_config *config,
                      hash[0], hash[1], hash[2], hash[3],
                      hash[4], hash[5], hash[6], hash[7]);
             if (ctx->suppress_type_name) {
-                index_len = snprintf(j_index,
-                                     ES_BULK_HEADER,
-                                     ES_BULK_INDEX_FMT_ID_WITHOUT_TYPE,
-                                     es_index,  es_uuid);
+                index_len = flb_sds_snprintf(&j_index,
+                                             flb_sds_alloc(j_index),
+                                             ES_BULK_INDEX_FMT_ID_WITHOUT_TYPE,
+                                             ctx->es_action,
+                                             es_index,  es_uuid);
             }
             else {
-                index_len = snprintf(j_index,
-                                     ES_BULK_HEADER,
-                                     ES_BULK_INDEX_FMT_ID,
-                                     es_index, ctx->type, es_uuid);
+                index_len = flb_sds_snprintf(&j_index,
+                                             flb_sds_alloc(j_index),
+                                             ES_BULK_INDEX_FMT_ID,
+                                             ctx->es_action,
+                                             es_index, ctx->type, es_uuid);
             }
         }
         if (ctx->ra_id_key) {
             id_key_str = es_get_id_value(ctx ,&map);
             if (id_key_str) {
                 if (ctx->suppress_type_name) {
-                    index_len = snprintf(j_index,
-                                         ES_BULK_HEADER,
-                                         ES_BULK_INDEX_FMT_ID_WITHOUT_TYPE,
-                                         es_index,  id_key_str);
+                    index_len = flb_sds_snprintf(&j_index,
+                                                 flb_sds_alloc(j_index),
+                                                 ES_BULK_INDEX_FMT_ID_WITHOUT_TYPE,
+                                                 ctx->es_action,
+                                                 es_index,  id_key_str);
                 }
                 else {
-                    index_len = snprintf(j_index,
-                                         ES_BULK_HEADER,
-                                         ES_BULK_INDEX_FMT_ID,
-                                         es_index, ctx->type, id_key_str);
+                    index_len = flb_sds_snprintf(&j_index,
+                                                 flb_sds_alloc(j_index),
+                                                 ES_BULK_INDEX_FMT_ID,
+                                                 ctx->es_action,
+                                                 es_index, ctx->type, id_key_str);
                 }
                 flb_sds_destroy(id_key_str);
                 id_key_str = NULL;
@@ -529,25 +560,42 @@ static int elasticsearch_format(struct flb_config *config,
         out_buf = flb_msgpack_raw_to_json_sds(tmp_sbuf.data, tmp_sbuf.size);
         msgpack_sbuffer_destroy(&tmp_sbuf);
         if (!out_buf) {
-            msgpack_unpacked_destroy(&result);
+            flb_log_event_decoder_destroy(&log_decoder);
             es_bulk_destroy(bulk);
+            flb_sds_destroy(j_index);
             return -1;
         }
 
+        out_buf_len = flb_sds_len(out_buf);
+        if (strcasecmp(ctx->write_operation, FLB_ES_WRITE_OP_UPDATE) == 0) {
+            tmp_buf = out_buf;
+            out_buf = flb_sds_create_len(NULL, out_buf_len = out_buf_len + sizeof(ES_BULK_UPDATE_OP_BODY) - 2);
+            out_buf_len = snprintf(out_buf, out_buf_len, ES_BULK_UPDATE_OP_BODY, tmp_buf);
+            flb_sds_destroy(tmp_buf);
+        }
+        else if (strcasecmp(ctx->write_operation, FLB_ES_WRITE_OP_UPSERT) == 0) {
+            tmp_buf = out_buf;
+            out_buf = flb_sds_create_len(NULL, out_buf_len = out_buf_len + sizeof(ES_BULK_UPSERT_OP_BODY) - 2);
+            out_buf_len = snprintf(out_buf, out_buf_len, ES_BULK_UPSERT_OP_BODY, tmp_buf);
+            flb_sds_destroy(tmp_buf);
+        }
+
         ret = es_bulk_append(bulk, j_index, index_len,
-                             out_buf, flb_sds_len(out_buf),
+                             out_buf, out_buf_len,
                              bytes, off_prev);
         flb_sds_destroy(out_buf);
+
         off_prev = off;
         if (ret == -1) {
             /* We likely ran out of memory, abort here */
-            msgpack_unpacked_destroy(&result);
+            flb_log_event_decoder_destroy(&log_decoder);
             *out_size = 0;
             es_bulk_destroy(bulk);
+            flb_sds_destroy(j_index);
             return -1;
         }
     }
-    msgpack_unpacked_destroy(&result);
+    flb_log_event_decoder_destroy(&log_decoder);
 
     /* Set outgoing data */
     *out_data = bulk->ptr;
@@ -563,7 +611,7 @@ static int elasticsearch_format(struct flb_config *config,
         fwrite(*out_data, 1, *out_size, stdout);
         fflush(stdout);
     }
-
+    flb_sds_destroy(j_index);
     return 0;
 }
 
@@ -619,7 +667,7 @@ static int elasticsearch_error_check(struct flb_elasticsearch *ctx,
      */
     /* Convert JSON payload to msgpack */
     ret = flb_pack_json(c->resp.payload, c->resp.payload_size,
-                        &out_buf, &out_size, &root_type);
+                        &out_buf, &out_size, &root_type, NULL);
     if (ret == -1) {
         /* Is this an incomplete HTTP Request ? */
         if (c->resp.payload_size <= 0) {
@@ -746,8 +794,8 @@ static int elasticsearch_error_check(struct flb_elasticsearch *ctx,
     return check;
 }
 
-static void cb_es_flush(const void *data, size_t bytes,
-                        const char *tag, int tag_len,
+static void cb_es_flush(struct flb_event_chunk *event_chunk,
+                        struct flb_output_flush *out_flush,
                         struct flb_input_instance *ins, void *out_context,
                         struct flb_config *config)
 {
@@ -758,9 +806,10 @@ static void cb_es_flush(const void *data, size_t bytes,
     size_t out_size;
     size_t b_sent;
     struct flb_elasticsearch *ctx = out_context;
-    struct flb_upstream_conn *u_conn;
+    struct flb_connection *u_conn;
     struct flb_http_client *c;
     flb_sds_t signature = NULL;
+    int compressed = FLB_FALSE;
 
     /* Get upstream connection */
     u_conn = flb_upstream_conn_get(ctx->u);
@@ -771,8 +820,9 @@ static void cb_es_flush(const void *data, size_t bytes,
     /* Convert format */
     ret = elasticsearch_format(config, ins,
                                ctx, NULL,
-                               tag, tag_len,
-                               data, bytes,
+                               event_chunk->type,
+                               event_chunk->tag, flb_sds_len(event_chunk->tag),
+                               event_chunk->data, event_chunk->size,
                                &out_buf, &out_size);
     if (ret != 0) {
         flb_upstream_conn_release(u_conn);
@@ -782,6 +832,29 @@ static void cb_es_flush(const void *data, size_t bytes,
     pack = (char *) out_buf;
     pack_size = out_size;
 
+    /* Should we compress the payload ? */
+    if (ctx->compress_gzip == FLB_TRUE) {
+        ret = flb_gzip_compress((void *) pack, pack_size,
+                                &out_buf, &out_size);
+        if (ret == -1) {
+            flb_plg_error(ctx->ins,
+                          "cannot gzip payload, disabling compression");
+        }
+        else {
+            compressed = FLB_TRUE;
+        }
+
+        /*
+         * The payload buffer is different than pack, means we must be free it.
+         */
+        if (out_buf != pack) {
+            flb_free(pack);
+        }
+
+        pack = (char *) out_buf;
+        pack_size = out_size;
+    }
+
     /* Compose HTTP Client request */
     c = flb_http_client(u_conn, FLB_HTTP_POST, ctx->uri,
                         pack, pack_size, NULL, 0, NULL, 0);
@@ -813,6 +886,11 @@ static void cb_es_flush(const void *data, size_t bytes,
     }
 #endif
 
+    /* Content Encoding: gzip */
+    if (compressed == FLB_TRUE) {
+        flb_http_set_content_encoding_gzip(c);
+    }
+
     /* Map debug callbacks */
     flb_http_client_debug(c, ctx->ins->callback);
 
@@ -847,12 +925,24 @@ static void cb_es_flush(const void *data, size_t bytes,
                 if (ctx->trace_error) {
                     /*
                      * If trace_error is set, trace the actual
-                     * input/output to Elasticsearch that caused the problem.
+                     * response from Elasticsearch explaining the problem.
+                     * Trace_Output can be used to see the request. 
                      */
-                    flb_plg_debug(ctx->ins, "error caused by: Input\n%s\n",
-                                  pack);
-                    flb_plg_error(ctx->ins, "error: Output\n%s",
-                                  c->resp.payload);
+                    if (pack_size < 4000) {
+                        flb_plg_debug(ctx->ins, "error caused by: Input\n%.*s\n",
+                                      (int) pack_size, pack);
+                    }
+                    if (c->resp.payload_size < 4000) {
+                        flb_plg_error(ctx->ins, "error: Output\n%s",
+                                      c->resp.payload);
+                    } else {
+                        /*
+                        * We must use fwrite since the flb_log functions
+                        * will truncate data at 4KB
+                        */
+                        fwrite(c->resp.payload, 1, c->resp.payload_size, stderr);
+                        fflush(stderr);
+                    }
                 }
                 goto retry;
             }
@@ -879,6 +969,11 @@ static void cb_es_flush(const void *data, size_t bytes,
  retry:
     flb_http_client_destroy(c);
     flb_free(pack);
+
+    if (out_buf != pack) {
+        flb_free(out_buf);
+    }
+
     flb_upstream_conn_release(u_conn);
     FLB_OUTPUT_RETURN(FLB_RETRY);
 }
@@ -921,6 +1016,13 @@ static struct flb_config_map config_map[] = {
      "Password for user defined in HTTP_User"
     },
 
+    /* HTTP Compression */
+    {
+     FLB_CONFIG_MAP_STR, "compress", NULL,
+     0, FLB_FALSE, 0,
+     "Set payload compression mechanism. Option available is 'gzip'"
+    },
+
     /* Cloud Authentication */
     {
      FLB_CONFIG_MAP_STR, "cloud_id", NULL,
@@ -960,6 +1062,17 @@ static struct flb_config_map config_map[] = {
      0, FLB_FALSE, 0,
      "External ID for the AWS IAM Role specified with `aws_role_arn`"
     },
+    {
+     FLB_CONFIG_MAP_STR, "aws_service_name", "es",
+     0, FLB_TRUE, offsetof(struct flb_elasticsearch, aws_service_name),
+     "AWS Service Name"
+    },
+    {
+     FLB_CONFIG_MAP_STR, "aws_profile", NULL,
+     0, FLB_TRUE, offsetof(struct flb_elasticsearch, aws_profile),
+     "AWS Profile name. AWS Profiles can be configured with AWS CLI and are usually stored in "
+     "$HOME/.aws/ directory."
+    },
 #endif
 
     /* Logstash compatibility */
@@ -976,6 +1089,11 @@ static struct flb_config_map config_map[] = {
      "become 'mydata-YYYY.MM.DD'. The last string appended belongs to the date "
      "when the data is being generated"
     },
+    {
+     FLB_CONFIG_MAP_STR, "logstash_prefix_separator", "-",
+     0, FLB_TRUE, offsetof(struct flb_elasticsearch, logstash_prefix_separator),
+     "Set a separator between logstash_prefix and date."
+    },
     {
      FLB_CONFIG_MAP_STR, "logstash_prefix_key", NULL,
      0, FLB_TRUE, offsetof(struct flb_elasticsearch, logstash_prefix_key),
@@ -1052,6 +1170,11 @@ static struct flb_config_map config_map[] = {
      "When enabled, generate _id for outgoing records. This prevents duplicate "
      "records when retrying ES"
     },
+    {
+     FLB_CONFIG_MAP_STR, "write_operation", "create",
+     0, FLB_TRUE, offsetof(struct flb_elasticsearch, write_operation),
+     "Operation to use to write in bulk requests"
+    },
     {
      FLB_CONFIG_MAP_STR, "id_key", NULL,
      0, FLB_TRUE, offsetof(struct flb_elasticsearch, id_key),
@@ -1094,6 +1217,7 @@ struct flb_output_plugin out_es_plugin = {
     .cb_pre_run     = NULL,
     .cb_flush       = cb_es_flush,
     .cb_exit        = cb_es_exit,
+    .workers        = 2,
 
     /* Configuration */
     .config_map     = config_map,
