@@ -2,8 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2021 The Fluent Bit Authors
- *  Copyright (C) 2015-2018 Treasure Data Inc.
+ *  Copyright (C) 2015-2024 The Fluent Bit Authors
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -31,6 +30,11 @@
 #define FLB_ES_DEFAULT_TIME_KEYF  "%Y-%m-%dT%H:%M:%S"
 #define FLB_ES_DEFAULT_TAG_KEY    "flb-key"
 #define FLB_ES_DEFAULT_HTTP_MAX   "512k"
+#define FLB_ES_DEFAULT_HTTPS_PORT 443
+#define FLB_ES_WRITE_OP_INDEX     "index"
+#define FLB_ES_WRITE_OP_CREATE    "create"
+#define FLB_ES_WRITE_OP_UPDATE    "update"
+#define FLB_ES_WRITE_OP_UPSERT    "upsert"
 
 struct flb_elasticsearch {
     /* Elasticsearch index (database) and type (table) */
@@ -51,6 +55,7 @@ struct flb_elasticsearch {
     int has_aws_auth;
     char *aws_region;
     char *aws_sts_endpoint;
+    char *aws_profile;
     struct flb_aws_provider *aws_provider;
     struct flb_aws_provider *base_aws_provider;
     /* tls instances can't be re-used; aws provider requires a separate one */
@@ -58,6 +63,8 @@ struct flb_elasticsearch {
     /* one for the standard chain provider, one for sts assume role */
     struct flb_tls *aws_sts_tls;
     char *aws_session_name;
+    char *aws_service_name;
+    struct mk_list *aws_unsigned_headers;
 #endif
 
     /* HTTP Client Setup */
@@ -84,6 +91,7 @@ struct flb_elasticsearch {
 
     /* prefix */
     flb_sds_t logstash_prefix;
+    flb_sds_t logstash_prefix_separator;
 
     /* prefix key */
     flb_sds_t logstash_prefix_key;
@@ -101,10 +109,15 @@ struct flb_elasticsearch {
     int time_key_nanos;
 
 
+    /* write operation */
+    flb_sds_t write_operation;
+    /* write operation elasticsearch operation */
+    flb_sds_t es_action;
+
     /* id_key */
     flb_sds_t id_key;
     struct flb_record_accessor *ra_id_key;
-    
+
     /* include_tag_key */
     int include_tag_key;
     flb_sds_t tag_key;
@@ -114,6 +127,9 @@ struct flb_elasticsearch {
 
     struct flb_record_accessor *ra_prefix_key;
 
+    /* Compression mode (gzip) */
+    int compress_gzip;
+
     /* Upstream connection to the backend server */
     struct flb_upstream *u;
 
