@@ -2,8 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2021 The Fluent Bit Authors
- *  Copyright (C) 2015-2018 Treasure Data Inc.
+ *  Copyright (C) 2015-2024 The Fluent Bit Authors
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -20,9 +19,17 @@
 
 #include <fluent-bit/flb_output_plugin.h>
 #include <fluent-bit/flb_snappy.h>
+#include <fluent-bit/flb_gzip.h>
 #include <fluent-bit/flb_metrics.h>
 #include <fluent-bit/flb_kv.h>
 
+#ifdef FLB_HAVE_SIGNV4
+#ifdef FLB_HAVE_AWS
+#include <fluent-bit/flb_aws_credentials.h>
+#include <fluent-bit/flb_signv4.h>
+#endif
+#endif
+
 #include "remote_write.h"
 #include "remote_write_conf.h"
 
@@ -36,12 +43,13 @@ static int http_post(struct prometheus_remote_write_context *ctx,
     void *payload_buf = NULL;
     size_t payload_size = 0;
     struct flb_upstream *u;
-    struct flb_upstream_conn *u_conn;
+    struct flb_connection *u_conn;
     struct flb_http_client *c;
     struct mk_list *head;
     struct flb_config_map_val *mv;
     struct flb_slist_entry *key = NULL;
     struct flb_slist_entry *val = NULL;
+    flb_sds_t signature = NULL;
 
     /* Get upstream context and connection */
     u = ctx->u;
@@ -53,8 +61,23 @@ static int http_post(struct prometheus_remote_write_context *ctx,
     }
 
     /* Map payload */
-    ret = flb_snappy_compress((void *) body, body_len,
-                              &payload_buf, &payload_size);
+
+    if (strcasecmp(ctx->compression, "snappy") == 0) {
+        ret = flb_snappy_compress((void *) body, body_len,
+                                  (char **) &payload_buf,
+                                  &payload_size);
+    }
+    else if (strcasecmp(ctx->compression, "gzip") == 0) {
+        ret = flb_gzip_compress((void *) body, body_len,
+                                &payload_buf, &payload_size);
+    }
+    else {
+        payload_buf = (void *) body;
+        payload_size = body_len;
+
+        ret = 0;
+    }
+
     if (ret != 0) {
         flb_upstream_conn_release(u_conn);
 
@@ -97,6 +120,21 @@ static int http_post(struct prometheus_remote_write_context *ctx,
                         FLB_PROMETHEUS_REMOTE_WRITE_VERSION_LITERAL,
                         sizeof(FLB_PROMETHEUS_REMOTE_WRITE_VERSION_LITERAL) - 1);
 
+    if (strcasecmp(ctx->compression, "snappy") == 0) {
+        flb_http_add_header(c,
+                            "Content-Encoding",
+                            strlen("Content-Encoding"),
+                            "snappy",
+                            strlen("snappy"));
+    }
+    else if (strcasecmp(ctx->compression, "gzip") == 0) {
+        flb_http_add_header(c,
+                            "Content-Encoding",
+                            strlen("Content-Encoding"),
+                            "gzip",
+                            strlen("gzip"));
+    }
+
     /* Basic Auth headers */
     if (ctx->http_user && ctx->http_passwd) {
         flb_http_basic_auth(c, ctx->http_user, ctx->http_passwd);
@@ -113,6 +151,30 @@ static int http_post(struct prometheus_remote_write_context *ctx,
                             val->str, flb_sds_len(val->str));
     }
 
+#ifdef FLB_HAVE_SIGNV4
+#ifdef FLB_HAVE_AWS
+    /* AWS SigV4 headers */
+    if (ctx->has_aws_auth == FLB_TRUE) {
+        flb_plg_debug(ctx->ins, "signing request with AWS Sigv4");
+        signature = flb_signv4_do(c,
+                                  FLB_TRUE,  /* normalize URI ? */
+                                  FLB_TRUE,  /* add x-amz-date header ? */
+                                  time(NULL),
+                                  (char *) ctx->aws_region,
+                                  (char *) ctx->aws_service,
+                                  0, NULL,
+                                  ctx->aws_provider);
+
+        if (!signature) {
+            flb_plg_error(ctx->ins, "could not sign request with sigv4");
+            out_ret = FLB_RETRY;
+            goto cleanup;
+        }
+        flb_sds_destroy(signature);
+    }
+#endif
+#endif
+
     ret = flb_http_do(c, &b_sent);
     if (ret == 0) {
         /*
@@ -126,7 +188,8 @@ static int http_post(struct prometheus_remote_write_context *ctx,
          * - 205: Reset content
          *
          */
-        if (c->resp.status < 200 || c->resp.status > 205) {
+        if ((c->resp.status < 200 || c->resp.status > 205) &&
+            c->resp.status != 400) {
             if (ctx->log_response_payload &&
                 c->resp.payload && c->resp.payload_size > 0) {
                 flb_plg_error(ctx->ins, "%s:%i, HTTP status=%i\n%s",
@@ -139,15 +202,30 @@ static int http_post(struct prometheus_remote_write_context *ctx,
             }
             out_ret = FLB_RETRY;
         }
+        else if (c->resp.status == 400) {
+            /* Returned 400 status means unrecoverable. Immidiately
+             * returning as a error. */
+            if (ctx->log_response_payload &&
+                c->resp.payload && c->resp.payload_size > 0) {
+                flb_plg_error(ctx->ins, "%s:%i, HTTP status=%i\n%s",
+                              ctx->host, ctx->port,
+                              c->resp.status, c->resp.payload);
+            }
+            else {
+                flb_plg_error(ctx->ins, "%s:%i, HTTP status=%i",
+                              ctx->host, ctx->port, c->resp.status);
+            }
+            out_ret = FLB_ERROR;
+        }
         else {
             if (ctx->log_response_payload &&
                 c->resp.payload && c->resp.payload_size > 0) {
-                flb_plg_info(ctx->ins, "%s:%i, HTTP status=%i\n%s",
+                flb_plg_debug(ctx->ins, "%s:%i, HTTP status=%i\n%s",
                              ctx->host, ctx->port,
                              c->resp.status, c->resp.payload);
             }
             else {
-                flb_plg_info(ctx->ins, "%s:%i, HTTP status=%i",
+                flb_plg_debug(ctx->ins, "%s:%i, HTTP status=%i",
                              ctx->host, ctx->port,
                              c->resp.status);
             }
@@ -159,6 +237,7 @@ static int http_post(struct prometheus_remote_write_context *ctx,
         out_ret = FLB_RETRY;
     }
 
+cleanup:
     /*
      * If the payload buffer is different than incoming records in body, means
      * we generated a different payload and must be freed.
@@ -204,8 +283,8 @@ static void append_labels(struct prometheus_remote_write_context *ctx,
     }
 }
 
-static void cb_prom_flush(const void *data, size_t bytes,
-                          const char *tag, int tag_len,
+static void cb_prom_flush(struct flb_event_chunk *event_chunk,
+                          struct flb_output_flush *out_flush,
                           struct flb_input_instance *ins, void *out_context,
                           struct flb_config *config)
 {
@@ -213,7 +292,7 @@ static void cb_prom_flush(const void *data, size_t bytes,
     int ok;
     int ret;
     int result;
-    cmt_sds_t encoded_chunk;
+    cfl_sds_t encoded_chunk;
     flb_sds_t buf = NULL;
     size_t diff = 0;
     size_t off = 0;
@@ -226,17 +305,20 @@ static void cb_prom_flush(const void *data, size_t bytes,
     result = FLB_OK;
 
     /* Buffer to concatenate multiple metrics contexts */
-    buf = flb_sds_create_size(bytes);
+    buf = flb_sds_create_size(event_chunk->size);
     if (!buf) {
         flb_plg_error(ctx->ins, "could not allocate outgoing buffer");
         FLB_OUTPUT_RETURN(FLB_RETRY);
     }
 
-    flb_plg_debug(ctx->ins, "cmetrics msgpack size: %lu", bytes);
+    flb_plg_debug(ctx->ins, "cmetrics msgpack size: %lu",
+                  event_chunk->size);
 
     /* Decode and encode every CMetric context */
     diff = 0;
-    while ((ret = cmt_decode_msgpack_create(&cmt, (char *) data, bytes, &off)) == ok) {
+    while ((ret = cmt_decode_msgpack_create(&cmt,
+                                            (char *) event_chunk->data,
+                                            event_chunk->size, &off)) == ok) {
         /* append labels set by config */
         append_labels(ctx, cmt);
 
@@ -266,7 +348,9 @@ static void cb_prom_flush(const void *data, size_t bytes,
         flb_plg_debug(ctx->ins, "final payload size: %lu", flb_sds_len(buf));
         if (buf && flb_sds_len(buf) > 0) {
             /* Send HTTP request */
-            result = http_post(ctx, buf, flb_sds_len(buf), tag, tag_len);
+            result = http_post(ctx, buf, flb_sds_len(buf),
+                               event_chunk->tag,
+                               flb_sds_len(event_chunk->tag));
 
             /* Debug http_post() result statuses */
             if (result == FLB_OK) {
@@ -328,6 +412,27 @@ static struct flb_config_map config_map[] = {
      0, FLB_TRUE, offsetof(struct prometheus_remote_write_context, http_passwd),
      "Set HTTP auth password"
     },
+    {
+     FLB_CONFIG_MAP_STR, "compression", "snappy",
+     0, FLB_TRUE, offsetof(struct prometheus_remote_write_context, compression),
+     "Compress the payload with either snappy, gzip if set"
+    },
+
+#ifdef FLB_HAVE_SIGNV4
+#ifdef FLB_HAVE_AWS
+    {
+     FLB_CONFIG_MAP_BOOL, "aws_auth", "false",
+     0, FLB_TRUE, offsetof(struct prometheus_remote_write_context, has_aws_auth),
+     "Enable AWS SigV4 authentication"
+    },
+    {
+     FLB_CONFIG_MAP_STR, "aws_service", "aps",
+     0, FLB_TRUE, offsetof(struct prometheus_remote_write_context, aws_service),
+     "AWS destination service code, used by SigV4 authentication"
+    },
+    FLB_AWS_CREDENTIAL_BASE_CONFIG_MAP(FLB_PROMETHEUS_REMOTE_WRITE_CREDENTIAL_PREFIX),
+#endif
+#endif
     {
      FLB_CONFIG_MAP_SLIST_1, "header", NULL,
      FLB_CONFIG_MAP_MULT, FLB_TRUE, offsetof(struct prometheus_remote_write_context, headers),
@@ -356,5 +461,6 @@ struct flb_output_plugin out_prometheus_remote_write_plugin = {
     .cb_exit     = cb_prom_exit,
     .config_map  = config_map,
     .event_type  = FLB_OUTPUT_METRICS,
+    .workers     = 2,
     .flags       = FLB_OUTPUT_NET | FLB_IO_OPT_TLS,
 };
