@@ -72,6 +72,7 @@
  */
 #define STACKDRIVER_NET_ERROR  502
 
+#define K8S_CLUSTER   "k8s_cluster"
 #define K8S_CONTAINER "k8s_container"
 #define K8S_NODE      "k8s_node"
 #define K8S_POD       "k8s_pod"
@@ -207,6 +208,9 @@ struct flb_stackdriver {
     /* upstream context for metadata end-point */
     struct flb_upstream *metadata_u;
 
+    /* the key to extract unstructured text payload from */
+    flb_sds_t text_payload_key;
+
 #ifdef FLB_HAVE_METRICS
     /* metrics */
     struct cmt_counter *cmt_successful_requests;
