@@ -2,8 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2021 The Fluent Bit Authors
- *  Copyright (C) 2015-2018 Treasure Data Inc.
+ *  Copyright (C) 2015-2024 The Fluent Bit Authors
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -21,9 +20,14 @@
 #ifndef FLB_OUT_TCP_H
 #define FLB_OUT_TCP_H
 
+#include <fluent-bit/flb_info.h>
+#include <fluent-bit/flb_record_accessor.h>
+
 struct flb_out_tcp {
     /* Output format */
     int out_format;
+    flb_sds_t raw_message_key;
+    struct flb_record_accessor *ra_raw_message_key;
 
     char *host;
     int port;
