@@ -2,7 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2021 The Fluent Bit Authors
+ *  Copyright (C) 2015-2024 The Fluent Bit Authors
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -88,7 +88,7 @@ struct flb_aws_credentials *get_credentials_fn_ec2(struct flb_aws_provider
         return NULL;
     }
 
-    creds = flb_malloc(sizeof(struct flb_aws_credentials));
+    creds = flb_calloc(1, sizeof(struct flb_aws_credentials));
     if (!creds) {
         flb_errno();
         return NULL;
@@ -158,7 +158,7 @@ void sync_fn_ec2(struct flb_aws_provider *provider) {
 
     flb_debug("[aws_credentials] Sync called on the EC2 provider");
     /* remove async flag */
-    implementation->client->upstream->flags &= ~(FLB_IO_ASYNC);
+    flb_stream_disable_async_mode(&implementation->client->upstream->base);
 }
 
 void async_fn_ec2(struct flb_aws_provider *provider) {
@@ -166,7 +166,7 @@ void async_fn_ec2(struct flb_aws_provider *provider) {
 
     flb_debug("[aws_credentials] Async called on the EC2 provider");
     /* add async flag */
-    implementation->client->upstream->flags |= FLB_IO_ASYNC;
+    flb_stream_enable_async_mode(&implementation->client->upstream->base);
 }
 
 void upstream_set_fn_ec2(struct flb_aws_provider *provider,
@@ -229,6 +229,8 @@ struct flb_aws_provider *flb_ec2_provider_create(struct flb_config *config,
         return NULL;
     }
 
+    pthread_mutex_init(&provider->lock, NULL);
+
     implementation = flb_calloc(1, sizeof(struct flb_aws_provider_ec2));
 
     if (!implementation) {
@@ -248,7 +250,10 @@ struct flb_aws_provider *flb_ec2_provider_create(struct flb_config *config,
         return NULL;
     }
 
-    upstream->net.connect_timeout = FLB_AWS_CREDENTIAL_NET_TIMEOUT;
+    /* IMDSv2 token request will timeout if hops = 1 and running within container */
+    upstream->base.net.connect_timeout = FLB_AWS_IMDS_TIMEOUT;
+    upstream->base.net.io_timeout = FLB_AWS_IMDS_TIMEOUT;
+    upstream->base.net.keepalive = FLB_FALSE; /* On timeout, the connection is broken */
 
     implementation->client = generator->create();
     if (!implementation->client) {
