@@ -2,7 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2021 The Fluent Bit Authors
+ *  Copyright (C) 2015-2024 The Fluent Bit Authors
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -103,15 +103,21 @@ struct flb_aws_credentials *get_credentials_fn_profile(struct flb_aws_provider
         time(NULL) >= implementation->next_refresh)) {
         AWS_CREDS_DEBUG("Retrieving credentials for AWS Profile %s",
                         implementation->profile);
-        ret = refresh_credentials(implementation, FLB_FALSE);
-        if (ret < 0) {
-            AWS_CREDS_ERROR("Failed to retrieve credentials for AWS Profile %s",
-                            implementation->profile);
+        if (try_lock_provider(provider) == FLB_TRUE) {
+            ret = refresh_credentials(implementation, FLB_FALSE);
+            unlock_provider(provider);
+            if (ret < 0) {
+                AWS_CREDS_ERROR("Failed to retrieve credentials for AWS Profile %s",
+                                implementation->profile);
+                return NULL;
+            }
+        } else {
+            AWS_CREDS_WARN("Another thread is refreshing credentials, will retry");
             return NULL;
         }
     }
 
-    creds = flb_malloc(sizeof(struct flb_aws_credentials));
+    creds = flb_calloc(1, sizeof(struct flb_aws_credentials));
     if (!creds) {
         flb_errno();
         goto error;
@@ -152,15 +158,27 @@ error:
 int refresh_fn_profile(struct flb_aws_provider *provider)
 {
     struct flb_aws_provider_profile *implementation = provider->implementation;
+    int ret = -1;
     AWS_CREDS_DEBUG("Refresh called on the profile provider");
-    return refresh_credentials(implementation, FLB_FALSE);
+    if (try_lock_provider(provider) == FLB_TRUE) {
+        ret = refresh_credentials(implementation, FLB_FALSE);
+        unlock_provider(provider);
+        return ret;
+    }
+    return ret;
 }
 
 int init_fn_profile(struct flb_aws_provider *provider)
 {
     struct flb_aws_provider_profile *implementation = provider->implementation;
+    int ret = -1;
     AWS_CREDS_DEBUG("Init called on the profile provider");
-    return refresh_credentials(implementation, FLB_TRUE);
+    if (try_lock_provider(provider) == FLB_TRUE) {
+        ret = refresh_credentials(implementation, FLB_TRUE);
+        unlock_provider(provider);
+        return ret;
+    }
+    return ret;
 }
 
 /*
@@ -221,12 +239,11 @@ static struct flb_aws_provider_vtable profile_provider_vtable = {
     .upstream_set = upstream_set_fn_profile,
 };
 
-struct flb_aws_provider *flb_profile_provider_create()
+struct flb_aws_provider *flb_profile_provider_create(char* profile)
 {
     struct flb_aws_provider *provider = NULL;
     struct flb_aws_provider_profile *implementation = NULL;
     int result = -1;
-    char *profile = NULL;
 
     provider = flb_calloc(1, sizeof(struct flb_aws_provider));
 
@@ -235,6 +252,8 @@ struct flb_aws_provider *flb_profile_provider_create()
         goto error;
     }
 
+    pthread_mutex_init(&provider->lock, NULL);
+
     implementation = flb_calloc(1,
                                 sizeof(
                                 struct flb_aws_provider_profile));
@@ -266,8 +285,10 @@ struct flb_aws_provider *flb_profile_provider_create()
         goto error;
     }
 
-    /* AWS profile name */
-    profile = getenv(AWS_PROFILE);
+    /* AWS profile name. */
+    if (profile == NULL) {
+        profile = getenv(AWS_PROFILE);
+    }
     if (profile && strlen(profile) > 0) {
         goto set_profile;
     }
